<?php
/**
 * Class ECBase
 * Plugin base class for initiating all the required actions, scripts and styles
 */

class ECBase
{
    private $template;
    public function __construct()
    {

        $this->options = ECOptions::get_instance();
        $this->erply   = ERPLYAPI::get_instance();

        // Register admin styles and scripts
        add_action('admin_print_styles', array($this, 'register_admin_styles'));
        add_action('admin_enqueue_scripts', array($this, 'register_admin_scripts'));

        // Register frontend styles
        add_action('wp_enqueue_scripts', array($this, 'register_frontend_styles'));
        add_action('wp_enqueue_scripts', array($this, 'register_frontend_scripts'));
		
		// Register admin notices
		add_action('admin_notices', array($this, 'display_admin_notices'));
		
        // Register menu
        add_action('admin_menu', array($this, 'register_plugin_menu'));

        // Register widgets
        add_action( 'widgets_init', array($this, 'register_widgets'));
        
        add_action( 'init', array($this, 'init'));
        add_action( 'plugins_loaded', array($this, 'load_first'));
    
        add_filter('the_content', array($this, 'manage_content'));
        add_action( 'parse_request', array($this, 'control_requests') );

        // Extend search
        add_action('posts_where_request', array(&$this, 'search'));

        // Template actions
        add_action('ec_before_content', array(&$this, 'action_before_content'));
        add_action('ec_after_content', array(&$this, 'action_after_content'));

        add_action( 'template_redirect', array(&$this, 'manage_templates'));
        add_action('pre_get_posts', array(&$this, 'pre_post'));

        add_action( 'wp_head', array($this, 'style_overrides' ));
        
        register_sidebar( array(
            'id'          => 'ec_front_sidebar',
            'name'        => __( 'eCommerce frontpage', EC_TEXTDOMAIN ),
            'description' => __( 'Frontpage widget area.', EC_TEXTDOMAIN )
        ) );

        $this->template = get_option('template');

        add_filter( 'manage_product_posts_columns', array(&$this, 'hide_product_columns'));
        add_action('pre_get_posts', array(&$this, 'pre_get'));

     } // end __construct()


    public function pre_get( $query ) {

        if(isset($query->query_vars['product-category'])){
            $query->query_vars['post_parent'] = 0;
        }

    }

    public function hide_product_columns( $columns )
    {
        unset($columns['comments']);
        unset($columns['author']);
        return $columns;
    }
    /*
    ======================================================================================
        Initialization
    ======================================================================================
    */
    public function pre_post( $query ) {
        global $wp_query, $wpdb;

        if(isset($query->query_vars['product-category'])){
            $query->query_vars['post_parent'] = 0;
        }

        if ($query->is_search) {
            $query->set('post_type', 'product');
        }


        //$wp_query->query_vars['no_found_rows'] = 1;
        // $wp_query->found_posts = $wpdb->get_var( "SELECT COUNT(*) FROM wp_posts WHERE 1=1 AND wp_posts.post_type = 'post' AND (wp_posts.post_status = 'publish' OR wp_posts.post_status = 'private')" );
        // $wp_query->found_posts = apply_filters_ref_array( 'found_posts', array( $wp_query->found_posts, &$wp_query ) );
        // $wp_query->max_num_pages = ceil($wp_query->found_posts / $wp_query->query_vars['posts_per_page']);

        return $wp_query;
    }

    public function load_first()
    {
        // Setup localization
        load_plugin_textdomain('erplycart', false, EC_RPATH . '/lang/');

        ECAjaxAdmin::get_instance();
        ECAjax::get_instance();
    } // end load_first()

    public function init()
    {
        // Settings Menu
        $this->settings = new ECSettings();
        // Initialize Cart
        $this->cart = new ECCart();   
        // User 
        $this->user = new ECUser();

        $this->shipping = new ECHandleShipping();
        $this->settings->set_shipping_modules($this->shipping->modules);

        $this->gateways = new ECHandleGateways();
        $this->settings->set_gateways($this->gateways->gateways);

        //Checkout
        $this->checkout = new ECCheckout($this->cart);
        $this->checkout->set_gateways($this->gateways->gateways);
        $this->checkout->set_shipping_modules($this->shipping->modules);

		//register taxonomy and custom post type
        $this->register_product_category();
        $this->register_product_post_type();
		
		global $pagenow;
		
		// Display login-details notice on dashboard
		if ($pagenow=='index.php') {
			$ec_login_settings = get_option('ec_login_settings');
			if (!$ec_login_settings['username'] || !$ec_login_settings['password'] || !$ec_login_settings['clientID']) {
				$notices= get_option('deferred_admin_notices', array());
				$notices[]= array('You have sucessfully installed eCommerce Shopping Cart plugin. Next step is to enter Inventory.com <strong><a href="admin.php?page=erply-cart-settings">account details here</a></strong>','notice');;
				update_option('deferred_admin_notices', $notices);
			}
		}
		
		if ($pagenow=='admin.php' && $_GET['page']=='erply-cart-settings') {
			
			// Display login-details notice on plugin settings page
			$ec_login_settings = get_option('ec_login_settings');
			if (!$ec_login_settings['username'] || !$ec_login_settings['password'] || !$ec_login_settings['clientID']) {
				$notices= get_option('deferred_admin_notices', array());
				$notices[]= array('Please enter Inventory.com account details below. If you do not have Inventory.com account you can <strong><a href="http://inventory.com/sign-up/?referer=wpplugin" target="_blank">sign up here</a></strong>. Trial account is free and it takes only few minutes to get started.','notice');;
				update_option('deferred_admin_notices', $notices);
			}
		
			// Get warehouses
			$response = $this->erply->sendRequest('getWarehouses', array());
			if (!empty($response->records)) {
				global $warehouses;
				$warehouses = $response->records;
			}
		
		}
		
		// Save login & Insert main settings
		if ($_POST['ec_login_settings']['username'] || $_POST['ec_login_settings']['password'] || $_POST['ec_login_settings']['clientID']) {
			if ($_POST['ec_login_settings']['username'] && $_POST['ec_login_settings']['password'] && $_POST['ec_login_settings']['clientID']) {
				$erplyApi = new ERPLY($_POST['ec_login_settings']['username'],$_POST['ec_login_settings']['password'],$_POST['ec_login_settings']['clientID']);
				$getCompanyInfo_response = $erplyApi->sendRequest('getCompanyInfo', array());
				if ($getCompanyInfo_response->records[0]->id) {
					$getWarehouses_response = $erplyApi->sendRequest('getWarehouses', array());
					update_option(
						'ec_main_settings',
						array(
							'defaultWarehouseID' => $getWarehouses_response->records[0]->warehouseID, 
							'defaultClientID' => $getCompanyInfo_response->records[0]->id, 
							'allowLogin' => 1, 
							'allowRegistration' => 1, 
							'overrideFrontPage' => 1, 
							'transactionType' => 'INVWAYBILL', 
							'welcomeMessage' => 'Welcome, %firstname%, %lastname%!'
						)
					);				
				} else {
					unset ($_POST['ec_login_settings']);
					delete_option('ec_login_settings');
					$notices = get_option('deferred_admin_notices', array());
					$notices[] = array('Could not connect to Inventory.com. Please check the login information.','error');
					update_option('deferred_admin_notices', $notices);
				}
			} else {
				unset ($_POST['ec_login_settings']);
				delete_option('ec_login_settings');
				$notices = get_option('deferred_admin_notices', array());
				$notices[] = array('Please fill in all fields.','error');
				update_option('deferred_admin_notices', $notices);
			}
		}
    }

    /*
    ======================================================================================
        Enqueue Stylesheets & Scripts
    ======================================================================================
    */

    /**
     * Register and enqueue backend stylesheets
     */
    public function register_admin_styles()
    {
        wp_enqueue_style('erply-cart-admin', EC_URL . '/assets/css/admin.css');
    } // end register_admin_styles()

    /**
     * Register and enqueue backend scripts
     */
    public function register_admin_scripts()
    {
        wp_enqueue_script( 'jquery' );
        wp_enqueue_script( 'erply-cart-admin', EC_URL . '/assets/js/admin.js', 'jquery' );
        $ajaxObject = array(
            'ajax_url' => admin_url( 'admin-ajax.php' ), 
            'nonce' => wp_create_nonce('ec-nonce')
        );
        wp_localize_script( 'erply-cart-admin', 'ecAjaxAdminObject', $ajaxObject, array($this, ));
    } // register_admin_scripts()

    /**
     * Register and enqueue frontend scripts
     */
    public function register_frontend_scripts()
    {
        wp_enqueue_script( 'jquery' );
        wp_enqueue_script( 'jquery-ui-tabs', 'jquery' );
        wp_enqueue_script('swipebox', EC_URL . '/assets/js/swipebox/jquery.swipebox.min.js', 'jquery');
        wp_enqueue_script('scrollbox', EC_URL . '/assets/js/jquery.scrollbox.js', 'jquery');
        wp_enqueue_script('cycle', EC_URL . '/assets/js/jquery.cycle2.min.js', 'jquery');
        wp_enqueue_script('erply-cart', EC_URL . '/assets/js/front.js', 'jquery');
        //wp_enqueue_script('erply-cart', EC_URL . '/assets/js/bootstrap.min.js', 'jquery');
        $ajaxObject = array(
            'ajax_url'   => admin_url( 'admin-ajax.php' ), 
            'nonce'      => wp_create_nonce('ec-nonce'),
            'customerID' => ec_logged_in()
        );
        wp_localize_script( 'erply-cart', 'ecAjaxObject', $ajaxObject, array($this, ));
    } // register_admin_scripts()

    /**
     * Register and enqueue frontend stylesheets
     */
    public function register_frontend_styles()
    {
        wp_register_style( 'erply-cart', EC_URL . '/assets/css/plugin.css',
        array(),
        EC_DEBUG == true ? time() : EC_VERSION, 
        'all' );
        wp_enqueue_style( 'erply-cart' );
        
        wp_register_style( 'swipebox', EC_URL . '/assets/js/swipebox/swipebox.css',
            array(),
            EC_VERSION,
            'all' );
        wp_enqueue_style( 'swipebox' );

        if($this->template == 'twentythirteen') {
            wp_register_style( 'erply-cart-t2013', EC_URL . '/assets/css/t2013.css',
            array(),
            EC_DEBUG == true ? time() : EC_VERSION, 
            'all' );
            wp_enqueue_style( 'erply-cart-t2013' );
        }
    } // end register_frontend_styles()

    public function style_overrides()
    {
        // dd($options);
        $css = isset($this->options->design['css']) ? $this->options->design['css'] : '';
        $css = trim($css);
        if(!empty($css)) {
            echo '<style type="text/css">' . "\n";   
            echo $css;
            echo '</style>' . "\n";
        }
    }

    /*
    ======================================================================================
        Register widgets
    ======================================================================================
    */



    public function register_widgets()
    {
         register_widget( 'ECCart_Widget' );
         register_widget( 'ECCategory_Widget' );
         //register_widget( 'ECTopProducts_Widget' );
         register_widget( 'ECLogin_Widget' );
         register_widget( 'ECFrontProducts_Widget' );
    }

    /*
    ======================================================================================
        Register Menu
    ======================================================================================
    */

    public function register_plugin_menu()
    {

        add_menu_page(__('eCommerce', EC_TEXTDOMAIN), __('eCommerce', EC_TEXTDOMAIN), 'manage_options', 'erply-cart', array($this, 'route'), null, null);
        global $submenu;
        if (isset($submenu['erply-cart']))
            $submenu['erply-cart'][0][0] = __('Dashboard', EC_TEXTDOMAIN);
    } // register_plugin_menu()


    /*
    ======================================================================================
        Register Poroduct PostType & Category
    ======================================================================================
    */

    /**
     * Register Products post type
     */
    public function register_product_post_type()
    {
    	// Display 'Products' in menu
    	$ec_main_settings = get_option('ec_main_settings');
    	
        register_post_type(
            'product',
            array(
                'labels' => array(
                    'name' => __('Products',EC_TEXTDOMAIN),
                    'singular_name' => __('Product',EC_TEXTDOMAIN)
                ),
                'public' => true,
                'show_ui' => $ec_main_settings['showProductsSection'] ? true : false,
                'hierarchical' => false,
                'show_in_nav_menus' => true,
                'supports' => array(
                    'title',
                    'editor', // (content)
                    'author',
                    'thumbnail', // (featured image, current theme must also support post-thumbnails)
                    'excerpt',
                    'trackbacks',
                    'custom-fields',
                    'comments', // (also will see comment count balloon on edit screen)
                    'page-attributes', // (menu order, hierarchical must be true to show Parent option)
                    'post-formats' // add post formats, see Post Formats
                ),
                'menu_position' => 23
            )
        );
    } // register_product_post_type()

    public function register_product_category(){
        $args = array(
            'hierarchical' => true,
            'labels' => array(
                'name' => __('Product categories',EC_TEXTDOMAIN)
            ),
            'show_ui' => true,
            'show_admin_column' => true,
            'query_var' => true,
            'rewrite' => array('slug' => 'product-category')
        );
        register_taxonomy('product-category', array('product'), $args);
    }

    /*
    ======================================================================================
        Populate Pages
    ======================================================================================
    */

    public function populate_pages()
    {
        $this->create_page( __('Cart', EC_TEXTDOMAIN), 'cartPage');
        $this->create_page( __('Checkout', EC_TEXTDOMAIN), 'checkoutPage');
        $this->create_page( __('Pay', EC_TEXTDOMAIN), 'payPage');
        $this->create_page( __('Thank You', EC_TEXTDOMAIN), 'thankyouPage');
        $this->create_page( __('Terms and Conditions', EC_TEXTDOMAIN), 'termPage');
        $this->create_page( __('Register new user', EC_TEXTDOMAIN), 'registerPage');
        $this->create_page( __('User orders history', EC_TEXTDOMAIN), 'historyPage');
    }

    private function create_page( $name, $setting )
    {
        $pageID = isset($this->options->page[$setting]) ? $this->options->page[$setting] : null;
        $pageCheck = !empty($pageID) ? get_page($pageID) : null;
        if(empty($pageID) || empty($pageCheck)) {
            $my_post = array(
              'post_title'    => $name,
              'post_type' => 'page',
              'post_status'   => 'publish',
              'comment_status' => 'closed'
            );
            
            $post = wp_insert_post( $my_post, true );
            if($post instanceof WP_Error) {
                var_dump($post);
            } else {
                $pagesSetting = (array)get_option( 'ec_pages_settings' );
                $pagesSetting[$setting] = $post;
                update_option( 'ec_pages_settings', $pagesSetting);
            }
        }
    }

    /*
    ======================================================================================
        Improve builtin search
    ======================================================================================
    */

    public function search($where)
    {
        if (is_search()) {
            global $wpdb, $wp;
            
            $where = preg_replace(
                "/($wpdb->posts.post_title (LIKE '%.*?%'))/i",
                "$0 OR ($wpdb->postmeta.meta_key = 'code' AND $wpdb->postmeta.meta_value $2)
                    OR ($wpdb->postmeta.meta_key = 'code2' AND $wpdb->postmeta.meta_value $2)
                ",
                $where
            );
            
            add_filter('posts_distinct_request', array(&$this, 'search_distinct'));
            add_filter('posts_join_request', array(&$this, 'search_join'));
        }
        return $where;
    }

    function search_join($join)
    {
        global $wp_query, $wpdb;

        if (!empty($wp_query->query_vars['s'])) {
            $join .= "LEFT JOIN $wpdb->postmeta ON $wpdb->posts.ID = $wpdb->postmeta.post_id ";
        }

        return $join;

    }

    function search_distinct($distinct) {
        $distinct = 'DISTINCT';
        return $distinct;
    }

    /*
    ======================================================================================
        Template hooks
    ======================================================================================
    */

    public function action_before_content($content){
        switch ($this->template)
        {
            case 'twentytwelve':
                $content .= '<div id="primary" class="site-content">
                             <div id="content" role="main">';
            break;
            case 'twentyeleven':
                $content .= '<div id="content" role="main">
				             <article class="post">';
            break;
            case 'twentythirteen':
                $content .= '<div id="primary" class="content-area">
		                     <div id="content" class="site-content" role="main">
		                     <article class="page type-page status-publish hentry">';
            break;
            default:
                $content .= '<div id="ec_content">';
            break;
        }

        echo $content;
    }

    public function action_after_content($content){
        switch ($this->template)
        {
            case 'twentytwelve':
                $content .= '</div><!-- #content -->
                             </div><!-- #primary -->';
                break;
            case 'twentyeleven':
                $content .= '</article><!-- .post -->
                             </div>';
                break;
            case 'twentythirteen':
                $content .= '</article><!-- .post -->
                             </div><!-- #content -->
                             </div><!-- #primary -->';
            break;
            default:
                $content .= '</div>';
                break;
        }

        echo $content;
    }

    /*
    ======================================================================================
        Replace content
    ======================================================================================
    */

    public function manage_content($content)
    {
        // Cart page
        $cartPageID = $this->options->page['cartPage'];
        if(is_page($cartPageID)) {
            $content = $this->cart->render();  
        }
        
        // Checkout page
        $checkoutPageID = $this->options->page['checkoutPage'];
        if(is_page($checkoutPageID)) {
            $this->checkout->render();
        }

        // Register page
        $registerPageID = $this->options->page['registerPage'];
        if(is_page($registerPageID))
            $content = $this->user->build_register_page();

        // Thank You page
        $thankyouPageID = $this->options->page['thankyouPage'];
        if(is_page($thankyouPageID))
            $content = $this->checkout->render_thankyou();

        // History page
        $historyPageID = $this->options->page['historyPage'];
        if(is_page($historyPageID))
            $content = $this->user->build_history_page();

        return $content;
    }

    function manage_templates()
    {
        $override = isset($this->options->main['overrideFrontPage']) ? $this->options->main['overrideFrontPage'] : false;
        if( is_front_page() && $override )
        {
            if ( '' === locate_template( array('front-page.php'), true, false ) )
                include( EC_PATH . '/templates/front-page.php' );
            exit();
        }
        if( is_search()){
            if ( '' === locate_template( array('taxonomy-product-category.php'), true, false ) )
                include( EC_PATH . '/templates/taxonomy-product-category.php' );
            exit();
        }
    }

    public function control_requests($arg)
    {
        // var_dump($_REQUEST);
        // die();
        if (isset($_REQUEST['ec_action'])) {
            $action = $_REQUEST['ec_action'];
            switch($action) {
                case 'ec_login':
                    $this->user->user_login();
                    break;
                case 'ec_logout':
                    $this->user->user_logout();
                    break;
                case 'ec_register':
                    $this->user->user_register();
                    break;
                case 'ec_checkout':
                    $this->checkout->new_order();
                    break;
                case 'ec_apply_coupon':
                    $this->cart->apply_coupon();
                    break;
                case 'ec_update_cart':
                    if(isset($_POST['update_cart'])) {
                        $this->cart->updateCart();
                    }
                    elseif(isset($_POST['proceed'])) {
                        $this->cart->updateCart();
                        wp_redirect( get_permalink( $this->options->page['checkoutPage'] ) );
                        exit;
                    } 
                                           
                    break;
                case 'ec_repeat_order':
                    $this->cart->repeatOrder();
                    break;
                case 'ec_sync':
                    ECCron::sync();
                    break;
            } //switch
        } // if
    }    

    public function create_termmeta_table()
    {
        global $wpdb;

        $table_name = $wpdb->prefix . "ec_termmeta";
          
        $sql = "CREATE TABLE $table_name (
        meta_id BIGINT(20) NOT NULL AUTO_INCREMENT,
        ec_term_id BIGINT(20) NOT NULL,
        meta_key VARCHAR(255) NULL DEFAULT NULL,
        meta_value LONGTEXT NULL,
        PRIMARY KEY  (meta_id)
        );";

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );
    } // end create_category_table()

    public function create_sessiondata_table()
    {
        global $wpdb;

        $table_name = $wpdb->prefix . "ec_sessions";

        $sql = "CREATE TABLE $table_name (
            session_id VARCHAR(255) NOT NULL,
            data TEXT NULL,
            user_agent VARCHAR(255) NULL DEFAULT NULL,
            flagged_for_update INT(11) NULL DEFAULT NULL,
            ip_address VARCHAR(50) NULL DEFAULT NULL,
            time_updated INT(8) NULL DEFAULT NULL,
            PRIMARY KEY (session_id)
        );";

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );
    } // end create_category_table()
	
	// Display admin notices
	public function display_admin_notices() {
		global $displayed_notices;
		if ($notices = get_option('deferred_admin_notices')) {
			foreach ($notices as $notice) {
				if (!isset($displayed_notices) || !in_array($notice[0],$displayed_notices)) {
					echo '
						<div class="updated" style="border:1px solid #'.($notice[1]=='error'?'C59A9A':'9AC59A').'; background:#'.($notice[1]=='error'?'FFEEEE':'EEFFEE').'; padding:10px; text-align:center; font-size:14px;">
							'.$notice[0].'
						</div>
					';
					$displayed_notices[] = $notice[0];
				}
			}			
			delete_option('deferred_admin_notices');
		}
	}	

}