<?php

class ECCart
{

    private $couponError;

    public function __construct()
    {
        $this->session = ECSessionHandler::get_instance();
        $this->options = ECOptions::get_instance();
        $this->erply = ERPLYAPI::get_instance();

        $this->couponError = '';

        $this->cart = (array)$this->session->getData('cart');
        $this->register_ajax_callbacks();
    }


    public function register_ajax_callbacks()
    {
        // Add to cart
        add_action('wp_ajax_ec_add_to_cart', array($this, 'addToCart'));
        add_action('wp_ajax_nopriv_ec_add_to_cart', array($this, 'addToCart'));

        // Remove from cart
        add_action('wp_ajax_ec_remove_from_cart', array($this, 'removeFromCart'));
        add_action('wp_ajax_nopriv_ec_remove_from_cart', array($this, 'removeFromCart'));

        // Get cart 
        add_action('wp_ajax_ec_get_cart', array($this, 'getCartHTML'));
        add_action('wp_ajax_nopriv_ec_get_cart', array($this, 'getCartHTML'));

        // Empty cart 
        add_action('wp_ajax_ec_empty_cart', array($this, 'emptyCart'));
        add_action('wp_ajax_nopriv_ec_empty_cart', array($this, 'emptyCart'));

        // Remove coupon
        add_action('wp_ajax_ec_remove_coupon', array($this, 'remove_coupon'));
        add_action('wp_ajax_nopriv_ec_remove_coupon', array($this, 'remove_coupon'));
    }


    public function addToCart()
    {
        //TODO error handling & sanitize
        $product = new ECProduct($_POST['id']);
        $amount = isset($_POST['amount']) && filter_var($_POST['amount'], FILTER_VALIDATE_INT) ? $_POST['amount'] : 1;

		if(!array_key_exists($product->ID, $this->cart)) 
			$this->cart[$product->ID] = array(
				'title' => $product->get_title(),
				'permalink' => $product->guid,
				'erplyID' => $product->get_erplyID(),
				'amount' => $amount
			);
        else
            $this->cart[$product->ID]['amount'] += $amount;

        do_action('ec_add_to_cart', $this->cart, $product, $amount);

        $this->session->setData('cart', $this->cart);

        die(json_encode(array('status' => 1, 'cartHTML' => $this->getCartHTML(true))));
    }

	public function updateCart()
	{
		$products = isset($_POST['products']) ? $_POST['products'] : array();
        $erplyIDs = false;

        if (empty($products) || !isset($products)) {
            $products = $_POST['erply_products'];
            $erplyIDs = true;
        }

        $this->cart = array();
        if (!empty($products)) {
            foreach ($products as $id => $amount) {
                //TODO out of the loop
                $product = new ECProduct($id, $erplyIDs);
                $this->cart[$id] = array(
                    'title' => $product->get_title(),
                    'permalink' => $product->guid,
                    'erplyID' => $product->get_erplyID(),
                    'amount' => filter_var($amount, FILTER_VALIDATE_INT) ? $amount : 1
                );
            }
            $this->session->setData('cart', $this->cart);
        }
    }

    public function removeFromCart()
    {
        $pID = $_POST['id'];
        if (array_key_exists($pID, $this->cart))
            unset($this->cart[$pID]);
        $this->session->setData('cart', $this->cart);
        die(json_encode(array('status' => 'ok')));
    }

    public function getCart()
    {
        return $this->cart;
    }

    public function get_promotions()
    {
        return $this->apply_promotions();
    }

    public function getCartHTML($return = false)
    {
        $view = new View('parts/ec-cart-widget.php');
        $view->cart = $this->cart;
        if($return) return $view->parse();
        $view->publish();
        die();
    }

    public function emptyCart()
    {
        $this->session->unsetData('cart');
        $this->session->unsetData('c_promotions');
        $this->session->unsetData('c_coupon');
        die();
    }

    public function apply_promotions()
    {
        $loginStatus = $this->session->getData('l_status');
        $couponRaw = $this->session->getData('c_coupon');
        $coupon = !empty($couponRaw) && is_array($couponRaw) ? implode(',', $couponRaw) : array();

        if (empty($this->cart)) return;

        $data = array(
            'warehouseID' => $this->options->main['defaultWarehouseID']
        );

        if (!empty($loginStatus))
            $data['customerID'] = $this->session->getData('l_cd_clientID');

        if (!empty($coupon))
            $data['couponIdentifiers'] = $coupon;

        $counter = 1;

        foreach ($this->cart as $productID => $product) {
            $data['productID' . $counter] = $product['erplyID'];
            $data['amount' . $counter] = $product['amount'];
            $counter++;
        }

        $response = $this->erply->sendRequest('calculateShoppingCart', $data);

        if ($response->status->errorCode == 0) {
            return $response->records[0];
        } else {
            ECLog::erply('calculateShoppingCart', $response->status->errorCode);
        }
        return null;
    }

    public function remove_coupon()
    {
        $key = $_POST['key'];
        $coupon = (array)$this->session->getData('c_coupon');

        if (!empty($coupon) && isset($coupon[$key]))
            unset($coupon[$key]);

        if (!empty($coupon))
            $this->session->setData('c_coupon', $coupon);
        else
            $this->session->unsetData('c_coupon');
        die();
    }

    public function apply_coupon()
    {
        $coupon = $_POST['ec_coupon'];
        $used_coupons = (array)$this->session->getData('c_coupon');

        if (empty($coupon)) $this->couponError = __('Please provide a coupon code.', EC_TEXTDOMAIN);

        $couponCheck = $this->verify_coupon($coupon);

        if ($couponCheck == 'ok') {
            if (empty($used_coupons) || (is_array($used_coupons) && !in_array($coupon, $used_coupons))) {
                $used_coupons[] = $coupon;
                $this->session->setData('c_coupon', $used_coupons);
            }
        } else {
            $this->couponError = $couponCheck;
        }
    } // end apply_coupon()

    public function verify_coupon($coupon)
    {
        $response = $this->erply->sendRequest('verifyIssuedCoupon', array('uniqueIdentifier' => $coupon));
        if ($response->status->errorCode == 0) {
            return 'ok';
        } else {
            if ($response->status->errorCode == 1040)
                return __('Such coupon has not been issued.', EC_TEXTDOMAIN);
            else if ($response->status->errorCode == 1041)
                return __('This coupon has already been redeemed.', EC_TEXTDOMAIN);
            else if ($response->status->errorCode == 1045)
                return __('This coupon has expired.', EC_TEXTDOMAIN);
        }
    }

    public function repeatOrder()
    {
        // TODO: Should add a notification if user has other items in the cart
        $this->updateCart();
        wp_redirect(get_permalink($this->options->page['cartPage']));
        exit;
    }

    public function render()
    {
        $promos = $this->apply_promotions();
        $view = new View('frontend/cart-page.tpl.php');
        $coupon = $this->session->getData('c_coupon');
        $loginStatus = $this->session->getData('l_status');

        if (!empty($this->cart) && !empty($promos)) {
            $view->cart = $this->cart;
            $view->promos = $promos;
            $view->loginData = $loginStatus == 'logged';
            $view->cartPermalink = get_permalink();
            $view->couponError = $this->couponError;
            $view->appliedCoupons = $coupon;
            $view->publish();
        } else {
            _e('Your cart is empty', EC_TEXTDOMAIN);
        }
    }
}