<?php
class ECCheckout
{
    public function __construct($cart)
    {
        $this->options = ECOptions::get_instance();
        $this->session = ECSessionHandler::get_instance();
        $this->cartObj = $cart;
        $this->errors = array();
        $this->erply = ERPLYAPI::get_instance();
        $this->sections = $this->register_fields();
        $this->cart = $this->cartObj->get_promotions();
    }

    public function set_gateways($gateways)
    {
        $this->gateways = $gateways;
    }

    public function set_shipping_modules($shipping)
    {
        $this->shipping = $shipping;
    }

    public function new_order()
    {
        $val = new Validator;
        $validationSource = array();
        foreach ($this->sections as $section) {
            foreach ($section['fields'] as $field) {
                $postedData = $_POST[ECForm::$prefix . $section['id']][$field['name']];
                $validationSource['sources'][$field['name']] = $postedData;
                $validationSource['labels'][$field['name']] = $field['title'];
                $validationSource['rules'][$field['name']] = array(
                    'type' => $field['validation']['type'],
                    'required' => $field['required'],
                    'min' => $field['validation']['min'],
                    'max' => $field['validation']['max'],
                    'trim' => $field['validation']['trim']
                );
            }
        }

        $val->addSource($validationSource['sources']);
        $val->addLabel($validationSource['labels']);
        $val->addRules($validationSource['rules']);
        $val->run();

        $shipping_method = isset($_POST['shipping_method']) ? $_POST['shipping_method'] : null;

        // Check if terms are accepted
        if (isset($_POST['terms']))
            $termsAccepted = $_POST['terms'];
        else {
            $this->errors['terms'] = __('Please accept the terms');
        }


        // No errors
        if (sizeof($val->errors) > 0 || !empty($this->errors)) {
            $this->errors = array_merge($this->errors, $val->errors);
        } else {

            $order = new ECOrder($this->sections, $this->cartObj, $shipping_method);
            $result = $order->new_order($val->sanitized);
            if (!empty($result)) {
                $this->session->unsetData('cart');
                $this->session->unsetData('c_promotions');
                $this->session->unsetData('c_coupon');
                $this->session->setData('last_order', $result);
                wp_redirect(get_permalink($this->options->page['thankyouPage']));
                exit;
            }
        }
    }

    public function prepare_gateways()
    {
        $gateways = array();

        foreach ($this->gateways as $gatewayName) {
            $gwID = 'ec_' . strtolower($gatewayName) . '_settings';
            $options = get_option($gwID);
            if (isset($options['enabled'])) {
                $gateway = new $gatewayName;
                $gateways[] = array(
                    'id' => $gwID,
                    'name' => $gateway->name,
                    'description' => isset($options['description']) ? $options['description'] : null
                );
            }
        }

        return $gateways;
    }

    public function prepare_shipping_modules()
    {
        $modules = array();

        foreach ($this->shipping as $shippingName) {
            $sID = 'ec_' . strtolower($shippingName) . '_settings';
            $options = get_option($sID);
            if (isset($options['enabled'])) {
                $modules[] = new $shippingName;
//				$shipping = new $shippingName;
//				$modules[] = array(
//					'id' => $sID,
//					'name' => $shipping->name,
//					'description' => isset($options['description']) ? $options['description'] : null
//				);
            }

        }

        return $modules;
    }

    public function render()
    {
        if (!empty($this->cart)) {
            $view = new View('frontend/checkout-page.tpl.php');
            // Send addresses to template if user is logged in
            if (ec_logged_in()) {
                $companyID = $this->session->getData('l_cd_companyID');
                $data = array(
                    'ownerID' => $companyID ? $companyID : ec_logged_in(),
                    'typeID' => 1
                );
                $response = $this->erply->sendRequest('getAddresses', $data);
                if (isset($response->status->responseStatus) && $response->status->responseStatus == 'ok') {
                    if (is_array($response->records) && count($response->records) > 1)
                        $view->availableAdresses = $response->records;
                }

                $selectedAddressID = $this->session->getData('l_cd_sa_addressID');
                if (isset($selectedAddressID)) {
                    $view->selectedAddress = $selectedAddressID;
                }
            }
            $view->sections = $this->sections;
            $view->cart = $this->cart;
            $view->errors = $this->errors;
            $view->gateways = $this->prepare_gateways();
            $view->shipping = $this->prepare_shipping_modules();
            $view->checkoutPage = $this->options->page['checkoutPage'];
            $view->termPage = $this->options->page['termPage'];
            $view->publish();
        } else {
            _e('Your cart is empty', EC_TEXTDOMAIN);
        }

    }

    public function render_thankyou()
    {
        $response = array();
        $lastOrder = $this->session->getData('last_order');
        if (!empty($lastOrder)) {
            $data = array('id' => $lastOrder);
            $response = $this->erply->sendRequest('getSalesDocuments', $data);
        }

        $view = new View('frontend/thankyou-page.tpl.php');
        $view->historyPage = $this->options->page['historyPage'];
        $view->lastOrderLink = isset($response->records[0]->invoiceLink) ? $response->records[0]->invoiceLink : '';
        $view->publish();
    }


    public function register_fields()
    {
        $userCreationFields = array(
            'username' => array(
                'type' => 'text',
                'title' => __('Username', EC_TEXTDOMAIN),
                'name' => 'username',
                'required' => false,
                'erply_method' => 'saveCustomer',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 20,
                    'trim' => true
                )
            ),
            'password' => array(
                'type' => 'password',
                'title' => __('Password', EC_TEXTDOMAIN),
                'name' => 'password',
                'required' => false,
                'erply_method' => 'saveCustomer',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 20,
                    'trim' => true
                )
            ),
            'password2' => array(
                'type' => 'password',
                'title' => __('Re-enter Password', EC_TEXTDOMAIN),
                'name' => 'password2',
                'required' => false,
                'erply_method' => 'saveCustomer',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 20,
                    'trim' => true
                )
            )
        );

        $userFields = array(
            'firstName' => array(
                'type' => 'text',
                'title' => __('First Name', EC_TEXTDOMAIN),
                'name' => 'firstName',
                'required' => true,
                'erply_method' => 'saveCustomer',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            ),
            'lastName' => array(
                'type' => 'text',
                'title' => __('Last Name', EC_TEXTDOMAIN),
                'name' => 'lastName',
                'required' => true,
                'erply_method' => 'saveCustomer',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            ),
            'email' => array(
                'type' => 'text',
                'title' => __('E-mail', EC_TEXTDOMAIN),
                'name' => 'email',
                'required' => true,
                'erply_method' => 'saveCustomer',
                'validation' => array(
                    'type' => 'email',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            ),
            'phone' => array(
                'type' => 'text',
                'title' => __('Phone', EC_TEXTDOMAIN),
                'name' => 'phone',
                'required' => true,
                'erply_method' => 'saveCustomer',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            ),
            'companyName' => array(
                'type' => 'text',
                'title' => __('Company Name', EC_TEXTDOMAIN),
                'name' => 'companyName',
                'required' => false,
                'erply_method' => 'saveCustomer',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            ),
            'companyID' => array(
                'type' => 'hidden',
                'title' => __('Company ID', EC_TEXTDOMAIN),
                'name' => 'companyID',
                'required' => false,
                'erply_method' => 'saveCustomer',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            ),
            'customerID' => array(
                'type' => 'hidden',
                'title' => __('Customer ID', EC_TEXTDOMAIN),
                'name' => 'customerID',
                'required' => false,
                'erply_method' => 'saveCustomer',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            )
        );

        $shippingFields = array(
            'street' => array(
                'type' => 'text',
                'title' => __('Address', EC_TEXTDOMAIN),
                'name' => 'street',
                'required' => true,
                'erply_method' => 'saveAddress',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            ),
            'address2' => array(
                'type' => 'text',
                'title' => __('Address 2', EC_TEXTDOMAIN),
                'name' => 'address2',
                'required' => false,
                'erply_method' => 'saveAddress',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            ),
            'city' => array(
                'type' => 'text',
                'title' => __('City', EC_TEXTDOMAIN),
                'name' => 'city',
                'required' => true,
                'erply_method' => 'saveAddress',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            ),
            'postalCode' => array(
                'type' => 'text',
                'title' => __('Postal Code', EC_TEXTDOMAIN),
                'name' => 'postalCode',
                'required' => true,
                'erply_method' => 'saveAddress',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            ),
            'country' => array(
                'type' => 'text',
                'title' => __('Country', EC_TEXTDOMAIN),
                'name' => 'country',
                'required' => true,
                'erply_method' => 'saveAddress',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            ),
            'state' => array(
                'type' => 'text',
                'title' => __('State', EC_TEXTDOMAIN),
                'name' => 'state',
                'required' => true,
                'erply_method' => 'saveAddress',
                'validation' => array(
                    'type' => 'string',
                    'min' => 1,
                    'max' => 255,
                    'trim' => true
                )
            )
        );
        if (isset($this->options->main['region'])) {
            if ($this->options->main['region'] != 'US' &&
                $this->options->main['region'] != 'CA' &&
                $this->options->main['region'] != 'MX'
            )
                unset($shippingFields['state']);
        }

        $sections = array(
            'personal_info' => array(
                'name' => __('Personal information', EC_TEXTDOMAIN),
                'id' => 'personal_info',
                'desc' => '',
                'collapsed' => false,
                'fields' => $userFields
            ),
            'create_user' => array(
                'name' => __('User Creation', EC_TEXTDOMAIN),
                'id' => 'create_user',
                'desc' => __('Would you like to register?', EC_TEXTDOMAIN),
                'collapsed' => true,
                'fields' => $userCreationFields
            ),
            'shipping' => array(
                'name' => __('Shipping', EC_TEXTDOMAIN),
                'id' => 'shipping',
                'desc' => __('Please fill out required information', EC_TEXTDOMAIN),
                'collapsed' => false,
                'fields' => $shippingFields,
            )
        );

        $customerID = ec_logged_in() ? ec_logged_in() : array();

        if (isset($_POST['ec_action']) && $_POST['ec_action'] == 'ec_checkout')
            return $this->refill_fields($sections, $customerID);
        elseif (!empty($customerID))
            return $this->fill_user_fields($sections, $customerID);

        return $sections;
    }

    public function refill_fields($sections, $customerID)
    {
        if (!empty($customerID)) unset($sections['create_user']);
        $pref = ECForm::$prefix;
        foreach ($sections as $section_key => $section) {
            if (!array_key_exists($pref . $section['id'], $_POST)) continue;
            foreach ($section['fields'] as $field_key => $field) {
                if (array_key_exists($field['name'], $_POST[$pref . $section['id']])) {
                    $sections[$section_key]['fields'][$field_key]['default'] = $_POST[$pref . $section['id']][$field['name']];
                }
            }
        }
        return $sections;
    }

    public function fill_user_fields($sections, $customerID)
    {
        unset($sections['create_user']);
        $companyID = $this->session->getData('l_cd_companyID');
        $companyName = $this->session->getData('l_cd_companyName');

        $data = array(
            'customerID' => $customerID == $companyID ? $this->session->getData('l_cd_clientID') : $customerID
        );

        $response = $this->erply->sendRequest('getCustomers', $data);

        if (!empty($response->records[0])) {
            if (isset($companyID)) { // If user is attached to a company use company addresses and show company name
                $response->records[0]->companyName = $companyName;
                $response->records[0]->companyID = $companyID;
            }
            foreach ($response->records[0] as $key => $value) {
                foreach ($sections['personal_info']['fields'] as $userFieldkey => $field) {
                    if ($field['name'] == $key) {
                        $sections['personal_info']['fields'][$userFieldkey]['default'] = $value;
                        break;
                    }
                }
            }
        }

        $selectedAddressID = $this->session->getData('l_cd_sa_addressID');
        $data = array(
            'ownerID' => $customerID,
            'typeID' => 1,
        );
        $response = $this->erply->sendRequest('getAddresses', $data);

        if (!empty($response->records)) {
            $index = 0;
            // We have a selected address in session find it from array
            if (isset($selectedAddressID) && !empty($selectedAddressID)) {
                foreach ($response->records as $record) {
                    if ($record->addressID == $selectedAddressID)
                        break;
                    $index++;
                }
            }

            foreach ($response->records[$index] as $key => $value) {
                foreach ($sections['shipping']['fields'] as $shippingFieldKey => $field) {
                    if ($field['name'] == $key) {
                        $sections['shipping']['fields'][$shippingFieldKey]['default'] = $value;
                        break;
                    }
                }
            }
        }

        return $sections;
    }

}