<?php
class ECOrder
{

    public function __construct($sections, $cart, $shipping_method)
    {
        $this->options = ECOptions::get_instance();
        $this->session = ECSessionHandler::get_instance();
        $this->erply = ERPLYAPI::get_instance();

        $this->customerID = null;
        $this->addressID = null;
        $this->sections = $sections;
        $this->cart = $cart;
        $this->shipping_method = $shipping_method;
    }

    public function get_address_id($clientID, $address = false)
    {
        // If an address is selected in session
        $selectedAddressID = $this->session->getData('l_cd_sa_addressID');
        $selectedAddressStreet = '';
        // Select first address for user by default

        $data = array(
            'ownerID' => $clientID
        );
        $response = $this->erply->sendRequest('getAddresses', $data);

        // We have a selected address in session find it from array
        if (!empty($response) && isset($response->records[0])) {

            $index = 0;
            if (isset($selectedAddressID) && !empty($selectedAddressID)) {

                foreach ($response->records as $key => $record) {
                    if( $record->typeID == 3 ) $this->payerAddressID = $record->addressID;
                    if ($record->addressID == $selectedAddressID) $index = $key;
                }
            }

            $selectedAddressID = $response->records[$index]->addressID;

            $selectedAddressStreet = $response->records[$index]->street;

        }

        if ($address == $selectedAddressStreet) {
            return $selectedAddressID;
        } else
            return $this->create_address($clientID);

    }

    public function parse_data($data)
    {
        $sections = $this->sections;
        $pref = ECForm::$prefix;

        $queryData = array();

        $errors = array();

        foreach ($sections as $section) {
            foreach ($section['fields'] as $field) {
                $queryData[$field['erply_method']][$field['name']] = $data[$field['name']];
            }
        }

        $this->queryData = $queryData;
    }

    private function get_shipping_method()
    {
        if (is_array($this->shipping_method) && array_key_exists('erply_shipping', $this->shipping_method)) {
            // erply shipping
            $params = array(
                'deliveryTypeID' => $this->shipping_method['erply_shipping']
            );
        } else {
            $params = array();
        }
        return $params;

    }

    private function save_customer()
    {
        $ec_session = ECSessionHandler::get_instance();

        $loggedInID = ec_logged_in();
        $customerID = $ec_session->getData('l_cd_clientID');
        $customer = $this->queryData['saveCustomer'];

        // Double query needed for company and for user
        if (isset($customer['companyName'])) { // Company name is set
            $companyData = array(
                'companyName' => $customer['companyName']
            );
            if (isset($customer['companyID']))
                $companyData['customerID'] = $customer['companyID'];
            $company = $this->erply->sendRequest('saveCustomer', $companyData);
            if ($company->status->errorCode === 0) {
                if (isset($company->records[0]->clientID)) {
                    $customer['companyID'] = $company->records[0]->clientID;
                    unset($customer['companyName']);
                    // Address for company
                    $this->queryData['saveAddress']['ownerID'] = $customer['companyID'];
                }
            } else {
                ECLog::erply('saveCustomer', $company->status->errorCode);
            }
        } // end if company
        if (!empty($loggedInID) && !empty($customerID))
            $customer['customerID'] = $customerID;

        // Update or create customer
        $response = $this->erply->sendRequest('saveCustomer', $customer);

        if (!empty($response->records)) { // API responded
            if (isset($customer['companyID']) && !empty($customer['companyID'])) {
                $returnID = $customer['companyID'];
            } else {
                $returnID = $response->records[0]->clientID;
                $this->queryData['saveAddress']['ownerID'] = $response->records[0]->clientID;
            }
        } // if API does not respond return null
        else
            $returnID = null;
        return $returnID;
    }

    private function create_address($customerID)
    {
        $customerAdress = $this->queryData['saveAddress'];
        $customerAdress['typeID'] = 1;
        $customerAdress['ownerID'] = $customerID;
        $response = $this->erply->sendRequest('saveAddress', $customerAdress);
        $addressID = $response->records[0]->addressID;
        return !empty($addressID) ? $addressID : null;
    }

    private function prepare_cart()
    {

        $cart = $this->cart->apply_promotions();
        $preparedCart = array();
        $counter = 1;
        foreach ($cart->rows as $item) {
            $preparedCart['productID' . $counter] = $item->productID;
            $preparedCart['vatrateID' . $counter] = 1;
            $preparedCart['amount' . $counter] = $item->amount;
            $preparedCart['price' . $counter] = $item->finalPrice;
            $counter++;
        }
        return $preparedCart;
    }

    public function new_order($queryData)
    {

        $transactionType = $this->options->main['transactionType'];
        $this->parse_data($queryData);

        $customerID = $this->save_customer();

        if (empty($customerID))
            return __('no customer', EC_TEXTDOMAIN);

        $shipping_method = $this->get_shipping_method();

        if (!empty($customerID))
            $addressID = $this->get_address_id($customerID, $this->queryData['saveAddress']['street']);
        else
            return __('no customer', EC_TEXTDOMAIN);

//		if(empty($shipping_method))
//			return 'no shipping_method';

        if (!empty($addressID)) {
            $cart = (array)$this->prepare_cart();

            if (empty($cart)) return;
            $saveSales = array(
                'type'           => $transactionType,
                'currencyCode'   => 'EUR',
                'customerID'     => $customerID,
                'addressID'      => $addressID,
                'paymentType'    => 'CARD'
            );

            if( isset($this->payerAddressID) ) {
                $saveSales['payerAddressID'] = $this->payerAddressID;
            }

            $saveSales = array_merge($saveSales, $cart, $shipping_method);
            $couponRaw = $this->session->getData('c_coupon');
            $coupon = !empty($couponRaw) && is_array($couponRaw) ? $couponRaw : array();
            $couponCheck = 'ok';
            if (!empty($coupon)) {
                foreach ($couponRaw as $couponIdentifier) {
                    $couponCheck = $this->cart->verify_coupon($couponIdentifier);
                    if ($couponCheck != 'ok') break;
                }
            }

            if ($couponCheck == 'ok') {
                $cInvoice = $this->erply->sendRequest('saveSalesDocument', $saveSales);
                if ($cInvoice->status->errorCode == 0) {
                    if (!empty($coupon))
                        foreach ($coupon as $couponIdentifier) {
                            $this->erply->sendRequest('redeemIssuedCoupon', array(
                                'uniqueIdentifier' => $couponIdentifier,
                                'timestamp' => time(),
                                'invoiceID' => $cInvoice->records[0]->invoiceID,
                                'customerID' => $customerID
                            ));
                        }
                    return $cInvoice->records[0]->invoiceID;
                } else {
                    ECLog::erply('saveSalesDocument', $cInvoice->status->errorCode);
                }
            } else {
                return $couponCheck;
            }


        } else {
            return 'no address';
        }

    }
}