<?php 
class ECProduct {

	private $promotedPrice, $erplyID;
	public function __construct($ID_or_Object, $searchByERPLYID = false)
	{
		$this->promotedPrice = null;
        $this->options = ECOptions::get_instance();
        $this->session = ECSessionHandler::get_instance();
        $this->erply = new ERPLY($this->options->login['username'], $this->options->login['password'], $this->options->login['clientID']);
		$post = null;
		
		if(is_object($ID_or_Object)) 
			$post = $ID_or_Object;
		elseif(!$searchByERPLYID) 
			$post = get_post($ID_or_Object);			
		else {
			$args = array( 
				'meta_key' => 'erplyID', 
				'meta_value' => $ID_or_Object,
				'post_type' => 'product'
			);
			$posts = get_posts( $args );
			if(!empty($posts))
				$post =  $posts[0];
		}

		if(!empty($post)) {
			foreach (get_object_vars($post) as $key => $value) {
	            $this->$key = $value;
	        }
		}
		if(!$post) throw new ECException(__('No Product Found',EC_TEXTDOMAIN), 1001);
	}	

	public function get_stock($force = false)
	{
		$user = ec_logged_in();
		$check = $force ? true : !empty($user);
		if($check) {
			$warehouseID = $this->options->main['defaultWarehouseID'];
            $productID = !isset($this->searchByERPLYID) ? get_post_meta($this->ID, 'erplyID', true) : $this->ID;

            $response = $this->erply->sendRequest('getProductStock', array('warehouseID' => $warehouseID, 'productID' => $productID));
	        if($response->status->errorCode == 0) {
		        $amount = $response->records[0]->amountInStock;
		        if($amount > 0) 
		        	return apply_filters( 'ec_product_stock', __('In Stock:', EC_TEXTDOMAIN) . ' ' . number_format($amount, 0, ',', ' '), $amount );
		        else 
		        	return apply_filters( 'ec_product_stock', __('Out of Stock', EC_TEXTDOMAIN), $amount );
	        } else {
	        	return __('Server request error, try again later.', EC_TEXTDOMAIN);
	        }
        } return null;
	} // end get_stock()

	public function get_title()
	{
		$title = !empty($this->post_title) ? $this->post_title : $this->post_excerpt;
		return apply_filters( 'ec_product_title', $title, $this );
	}

	public function get_thumb($size = 'thumb', $single = false)
	{
		$images = get_post_meta($this->ID, 'images', true);				
		if(!empty($images)):			
            // $image = get_image($images, $size);
            $image = get_gallery($images, $size, $single);
        else:
            $image = $this->get_category_images($size,$single);
        endif;
		return $image;
	}

	public function get_files()
	{
		$files = get_post_meta($this->ID, 'files', true);	
		return unserialize($files);
	}

	private function get_category_images($size,$single = false)
	{
		$categories = wp_get_post_terms( $this->ID, 'product-category');
		if(!empty($categories)) {
			$categoryID = $categories[0]->term_id;
			$images = $this->walk_categories($categoryID);
			if(!empty($images))
				return get_gallery($images, $size, $single);
		}
		return get_gallery(serialize(array()), $size, $single);
	} // end get_category_image()

	private function walk_categories($catID)
	{
		$term = get_term_by( 'id', $catID, 'product-category' );
        $images = ec_term_meta($term->term_id, 'images');
        if(!empty($images))
			return $images;
		else {
			if($term->parent != 0) 
				return $this->walk_categories($term->parent);
			else return null;
		}		
	} // end walk_categories()

	public function prepare_price()
	{
		$this->options = ECOptions::get_instance();
		$region = $this->options->main['region'];
		
		if(!empty($this->promotedPrice)){
			if($region != 'US')
				return array('current' => $this->promotedPrice->finalPriceWithVAT);
			else 
				return array('current' => $this->promotedPrice->finalPrice);
		}
		else {
            $loginCheck = $this->session->getData('l_status');
            $clientID = $this->session->getData('l_cd_clientID');

            if (!empty($loginCheck) && !empty($clientID)) {
				$data['customerID'] = $clientID;

				$data['productID1'] = $this->get_erplyID();
				$data['amount1'] = 1;
				
				$erply = ERPLYAPI::get_instance();
				$response = $erply->sendRequest('applyPromotions', $data);


				if($response->status->errorCode == 0) {
					$rows = (array)$response->records[0]->rows;
					$row = reset($rows);

					if($region != 'US'){
						return array(
                                    'current' => $row->finalPriceWithVAT,
                                    'original' => $row->originalPriceWithVAT
                        );
                    }
					else{
						return array(
                                    'current' => $row->finalPrice,
                                    'original' => $row->originalPrice
                        );
                    }
				}
			}
		} 

		$price = $region != 'US' ? 
			get_post_meta( $this->ID, 'priceWithVat', true) : 
			get_post_meta( $this->ID, 'price', true);
		return array('current' => $price);
	}

	public function set_promoted_price($priceObj)
	{
		$this->promotedPrice = $priceObj;
	}

	private function get_additional_products($type)	
	{
		$preparedProducts = array();
		$products = get_post_meta( $this->ID, $type, true);
		if (isset($products)) {
			$products = @unserialize($products);
			if($products !== false)
				foreach ($products as $product) {
					$preparedProducts[] = new ECProduct($product, true);
				}		
		}		

		$loginData = ec_logged_in();
        if (!empty($loginData) && !empty($preparedProducts)) {
        	$promoQuery = array();
        	$counter = 0;
        	foreach ($preparedProducts as $product) {
        		$erplyID = $product->get_erplyID();                    
	            $promoQuery['productID'.$counter] = $erplyID;
	            $promoQuery['amount'.$counter] = 1;
	            $counter++;
        	}
        	$rows = null;
            $promoQuery['customerID'] = $loginData;

            $erply = ERPLYAPI::get_instance();
            $response = $erply->sendRequest('applyPromotions', $promoQuery);


            if($response->status->responseStatus == 'ok')            
                 $rows = (array)$response->records[0]->rows;
       
			if(!empty($rows)) {
            	foreach ($preparedProducts as $key => $product) {	            
	                foreach ($rows as $row) {
	                    if($row->productID == $product->get_erplyID()) {
	                        $preparedProducts[$key]->set_promoted_price( $row ); 
	                        break;
	                    }
	                }
	            }
        	}
        }

		return $preparedProducts;
	}

	public function search_products()
    {
        global $wp_query;
        print_r($wp_query->query_vars);
    }

	public function get_erplyID()
	{
		if (!isset($this->erplyID)) 
			$this->erplyID = get_post_meta( $this->ID, 'erplyID', true);
		return $this->erplyID;
	}	

	public function get_price()
	{
		$priceArray = $this->prepare_price();
        $originalPrice = isset($priceArray['original']) ? $priceArray['original'] : false;
        $price = $priceArray['current'];

        $return = is_numeric($price) ? number_format($price, 2, ',', ' ') . ' €' : '';
        if($originalPrice && $originalPrice > $price)
            $return .= ' <del>'.number_format($originalPrice, 2, ',', ' ') . ' €</del>';

		return $return;
	}

	public function get_attributes()
	{
		$parameters = get_post_meta( $this->ID, 'parameters', true );
		$parameters = @unserialize($parameters);
		return ($parameters !== false) ? $parameters : null;
	}

	public function get_related_products() {
		return $this->get_additional_products('relatedProducts');
	}

	public function get_replacement_products() {
		return $this->get_additional_products('replacementProducts');
	}	

	public function get_attr($attrName)
	{
		return !empty($attrName) ? get_post_meta( $this->ID, $attrName, true) : null;
	} // end get_attr()

}