<?php
/**
 * Class ECSettings
 * Handles all the plugin settings
 */
class ECSettings {

    private $gateways;
    private $shipping;
    public function __construct() {
        // Register menu item
        add_action('admin_menu', array($this, 'register_menu_item'));
        add_action('admin_init', array($this, 'register_settings'));
    }

    /**
     * Load settings page view
     */
    public function view_index() {
        if ( isset( $_GET['page'] ) && 'erply-cart-settings' == $_GET['page'] )
            new View('settings.tpl.php', true);
    }

    public function set_shipping_modules($shipping)
    {
        $this->shipping = $shipping;
    }

    public function set_gateways($gateways)
    {
        $this->gateways = $gateways;
    }

    public function register_shipping_settings()
    {
        foreach ($this->shipping as $moduleName) {
            $shipping = new $moduleName;
            $sectionfields = array();

            $this->register_section(
                'ecs_'. $shipping->id .'_section',     // Section ID
                $shipping->name,                 // Title
                'erply-shipping-settings',  // Page ID
                'ec_'. $shipping->id .'_settings',     // Setting Name
                'ecs_shipping_section',     // Parent Section
                $shipping->get_fields(),     // Fields
                __( 'Options for', EC_TEXTDOMAIN ). ' ' . $shipping->name
            );

        }
    }

    public function register_gateway_settings()
    {
        foreach ($this->gateways as $gatewayName) {
            $gateway = new $gatewayName;
            $sectionfields = array();

            $this->register_section(
                'ecs_'. $gateway->id .'_section',     // Section ID
                $gateway->name,                 // Title
                'erply-gateway-settings',  // Page ID
                'ec_'. $gateway->id .'_settings',     // Setting Name
                'ecs_gateway_section',     // Parent Section
                $gateway->get_fields(),     // Fields
                __( 'Options for', EC_TEXTDOMAIN ). ' ' . $gateway->name
            );

        }
    }

    /**
     * Register settings
     */
    public function register_settings() {        
		
		// Options for 'Default Warehouse ID'
		global $warehouses;
		if ($warehouses) {
			foreach ($warehouses as $warehouse) {
				$defaultWarehouseID_options[$warehouse->warehouseID] = __($warehouse->name,EC_TEXTDOMAIN);
			}
		}
        
        $designSectionFields = array(
            array(
                'id' => 'css',
                'label' => __('CSS Overrides',EC_TEXTDOMAIN),
                'type' => 'textarea'
            )
        );

        $loginSectionFields = array(
            array(
                'id' => 'username',
                'label' => __('Username',EC_TEXTDOMAIN),
            ),
            array(
                'id' => 'password',
                'label' => __('Password',EC_TEXTDOMAIN)
            ),
            array(
                'id' => 'clientID',
                'label' => __('Client ID',EC_TEXTDOMAIN)
            )
        );

        $mainSectionfields = array(
            array(
                'id' => 'defaultWarehouseID',
                'label' => __('Default Warehouse ID',EC_TEXTDOMAIN),
                'type' => 'select',
                'options' => $defaultWarehouseID_options,
                'description' => __('This warehouse will be used as your web store warehouse. Product stock levels displayed to customers will be taken from that warehouse. All orders will be tied to that warehouse also.',EC_TEXTDOMAIN)
            ),
        		/*
            array(
                'id' => 'defaultClientID',
                'label' => __('Default Client ID',EC_TEXTDOMAIN),
                'default' => 1
            ),*/
            array(
                'id' => 'allowLogin',
                'label' => __('Allow user login',EC_TEXTDOMAIN),
                'type' => 'checkbox'
            ),
            array(
                'id' => 'allowRegistration',
                'label' => __('Allow user registration',EC_TEXTDOMAIN),
                'type' => 'checkbox'
            ),
            array(
                'id' => 'overrideFrontPage',
                'label' => __('Override front page',EC_TEXTDOMAIN),
                'type' => 'checkbox',
                'description' => __('Use fullscreen front page with a large widget area instead of default WordPress front page',EC_TEXTDOMAIN)
            ),
            array(
                'id' => 'transactionType',
                'label' => __('Orders are saved as',EC_TEXTDOMAIN),
                'type' => 'select',
                'options' => array(
                	'INVWAYBILL' => __('Invoice-waybills',EC_TEXTDOMAIN),
                 	'ORDER' => __('Orders',EC_TEXTDOMAIN)
                 ),
				'description' => __('You can choose a type of document created whan customer places order. This can depend on your payment or fulfillment process.',EC_TEXTDOMAIN)
            ),
            array(
                'id' => 'welcomeMessage',
                'label' => __('Welcome Message',EC_TEXTDOMAIN),
                'type' => 'textarea',
                'description' => __('Enter welome message for the loggged in user. You can use variables %firstname%, %lastname%.',EC_TEXTDOMAIN)
            ),
			array(
                'id' => 'showProductsSection',
                'label' => __('Display imported products in WordPress backend', EC_TEXTDOMAIN),
                'type' => 'checkbox',
                'description' => __('Warning! In normal situation you do not want enable this option, because managing products should be done in Inventory.com backend only. When enabling this option, you must consider that product information from Inventory.com will always overwrite any changes you make in WordPress.',EC_TEXTDOMAIN)
            ),
            /*array(
                'id' => 'lastSync',
                'label' => __('Last Update',EC_TEXTDOMAIN),
                'description' => 'Will be hidden in production version.'
            ),*/
            /*array(
                'id' => 'region',
                'label' => __('Region',EC_TEXTDOMAIN),
                'description' => 'Will be hidden in production version.'
            )*/
        );

        $pagesSectionfields = array(
            array(
                'id' => 'cartPage',
                'label' => __('Cart Page',EC_TEXTDOMAIN)
            ),
            array(
                'id' => 'checkoutPage',
                'label' => __('Checkout Page',EC_TEXTDOMAIN),
            ),
            array(
                'id' => 'termPage',
                'label' => __('Terms Page',EC_TEXTDOMAIN),
            ),
            array(
                'id' => 'thankyouPage',
                'label' => __('Thank You Page',EC_TEXTDOMAIN),
            ),
            array(
                'id' => 'registerPage',
                'label' => __('Register new user',EC_TEXTDOMAIN),
            ),
            array(
                'id' => 'historyPage',
                'label' => __('User orders history',EC_TEXTDOMAIN),
            )
        );


        $this->register_section(
            'ecs_main_section',     // Section ID
            __('Main',EC_TEXTDOMAIN),                 // Title
            'erply-main-settings',  // Page ID
            'ec_main_settings',     // Setting Name
            'ecs_main_section',     // Parent Section
            $mainSectionfields,     // Fields 
            __( 'General settings.', EC_TEXTDOMAIN )
        );

        $this->register_section(
            'ecs_login_section',    // Section ID
            __('Login',EC_TEXTDOMAIN),                // Title
            'erply-login-settings',  // Page ID
            'ec_login_settings',    // Setting Name
            'ecs_login_section',     // Parent Section
            $loginSectionFields,    // Fields
            __( 'Inventory.com account information.', EC_TEXTDOMAIN )
        );

        $this->register_section(
            'ecs_design_section',    // Section ID
            __('Design',EC_TEXTDOMAIN),                // Title
            'erply-design-settings',  // Page ID
            'ec_design_settings',    // Setting Name
            'ecs_design_section',     // Parent Section
            $designSectionFields,    // Fields
            __( 'eCommerce Styling.', EC_TEXTDOMAIN )
        );

        $this->register_section(
            'ecs_pages_section',    // Section ID
            __('Pages',EC_TEXTDOMAIN),                // Title
            'erply-pages-settings', // Page ID
            'ec_pages_settings',     // Setting Name
            'ecs_pages_section',     // Parent Section
            $pagesSectionfields,      // Fields
            __( 'You should only change these if your are re-installing the plugin and you want to use your existing pages.', EC_TEXTDOMAIN )      // Fields
        );
        
        $this->register_section(
            'ecs_gateway_section',
            __('Payment Gateways',EC_TEXTDOMAIN),
            'erply-gateway-settings',
            null,
            'ecs_gateway_section',
            array(),      // Fields
            __( 'These are just dummy payment gateways. You can customize the plugin to add your own gateways. We will be adding popular gateways in future versions of the plugin.', EC_TEXTDOMAIN )
        );

        $this->register_section(
            'ecs_shipping_section',
            __('Shipping Modules',EC_TEXTDOMAIN),
            'erply-shipping-settings',
            null,
            'ecs_shipping_section',
            array(),      // Fields
            __( 'These are just dummy shipping modules. You can customize the plugin to add your own module. We will be adding popular shipping options in future versions of the plugin.', EC_TEXTDOMAIN )
        );

        $this->register_gateway_settings();
        $this->register_shipping_settings();
    }

    public function register_section($name, $title, $page, $settingName, $parentSection, $fields, $sectionDescription)
    {
        add_settings_section(
            $name,
            $title,
            array($this, 'print_section_info'),
            $page
        );
        $this->$name = $sectionDescription;

        if(!empty($settingName)) {
            if(!empty($fields))
                $this->populate_fields($fields, $page, $name, $settingName);

            register_setting( $parentSection, $settingName);    
        }
        
    }

    public function populate_fields($fields, $page, $section, $setting)
    {
        foreach ($fields as $f) {
            add_settings_field(
                $f['id'],
                $f['label'],
                array($this, 'create_from_field'),
                $page,
                $section,
                array( 
                    'name' => $f['id'],  
                    'setting' => $setting,
                    'type' => isset($f['type']) ? $f['type'] : null,
                    'default' => isset($f['default']) ? $f['default'] : '',
                    'options' => isset($f['options']) ?$f['options'] : '',
                    'description' => isset($f['description']) ?$f['description'] : ''
                )
            );
        }

    }

    public function print_section_info($sectionID){
        echo $this->$sectionID['id'];
    }

    public function create_from_field($args){
        $option = get_option($args['setting']);
        $option = isset($option[$args['name']]) ? $option[$args['name']] : null;

        if($args['type'] == 'checkbox') : ?>
            <input type="checkbox" 
                name="<?=$args['setting'] ?>[<?= $args['name'] ?>]" 
                id="ecs_setting[<?= $args['name'] ?>]" 
                value="1" 
                <?php checked( 1, $option, true ) ?> />
        <?php elseif($args['type'] == 'select'): ?>
            <select name="<?=$args['setting'] ?>[<?= $args['name'] ?>]" id="ecs_setting[<?= $args['name'] ?>]">
                <?php foreach($args['options'] as $k => $v): ?>
                    <option value="<?php echo $k ?>" <?php selected( $option, $k); ?>><?php echo $v ?></option>
                <?php endforeach; ?>
            </select>
        <?php elseif($args['type'] == 'textarea'): ?>
            <textarea 
            style="height: 200px; width: 450px;"
            name="<?=$args['setting'] ?>[<?= $args['name'] ?>]" id="ecs_setting[<?= $args['name'] ?>]"><?=$option;?></textarea>
        <?php else: ?>
            <input type="text"
                id="ecs_setting[<?= $args['name'] ?>]"
                name="<?=$args['setting'] ?>[<?= $args['name'] ?>]" 
                value="<?=$option;?>" />
        <?php
        endif;

        echo !empty($args['description']) ? '<br><small>'. $args['description'].'</small>' : '';
    }

    public function register_menu_item() {
        add_submenu_page(
            'erply-cart', 
            __('eCommerce Settings', EC_TEXTDOMAIN), 
            __('Settings', EC_TEXTDOMAIN), 
            'manage_options', 
            'erply-cart-settings', 
            array($this, 'view_index')
        );
    } // register_menu_item()

    public function remove_settings()
    {
        //TODO    
    }
}