<?php 
class ECUser {

	public function __construct()
	{
        $this->options = ECOptions::get_instance();
        $this->erply = ERPLYAPI::get_instance();
        $this->session = ECSessionHandler::get_instance();

        // Array for registration errors and fields
        $this->registration_info = array();
	}

	public function build_register_page()
    {
        $view = new View('frontend/register-page.tpl.php');
        $view->info = $this->registration_info;
        return $view->parse();
   }

    public function build_history_page()
    {
        $view = new View('frontend/history-page.tpl.php');
        $view->orders = $this->get_user_orders();
        return $view->parse();
    }

	public function user_register()
    {
        // If nonce is not valid die !
        $nonce = $_REQUEST['_ec_nonce'];
        if(!wp_verify_nonce( $nonce, 'ec_register' ))
            die('Busted');

        $firstName = sanitize_text_field($_POST['user_first_name']);
        $lastName = sanitize_text_field($_POST['user_last_name']);
        $email = sanitize_email($_POST['user_email']);
        $username = sanitize_text_field($_POST['user_username']);
        $password = sanitize_text_field($_POST['user_password']);

        // Validation
        if(!isset($firstName) || empty($firstName) || strlen($firstName) < 2)
            $this->registration_info['errors']['user_first_name'] = true;

        if(!isset($lastName) || empty($lastName) || strlen($lastName) < 2)
            $this->registration_info['errors']['user_last_name'] = true;

        if(!isset($email) || empty($email))
            $this->registration_info['errors']['user_email'] = true;

        if(!isset($username) || empty($username)){
            $this->registration_info['errors']['user_username'] = true;
        }
        else{
            $uniqueData = array(
                'username' => $username
            );
            $uniqueResponse = $this->erply->sendRequest('validateCustomerUsername', $uniqueData);
            if(isset($uniqueResponse->status) && $uniqueResponse->status->responseStatus != 'ok'){ // Username is unique
                $this->registration_info['errors']['user_username'] = true;
            }
        }


        if(!isset($password) || empty($password))
            $this->registration_info['errors']['user_password'] = true;

        if(empty($this->registration_info['errors'])){
            $data = array(
                'firstName' => $firstName,
                'lastName' => $lastName,
                'email' => $email,
                'username' => $username,
                'password' => $password
            );

            $response = $this->erply->sendRequest('saveCustomer', $data);
            if(isset($response->status) && $response->status->responseStatus == 'ok'){ // New client created lets log in
                if(isset($response->records[0]->clientID)){
                    // Store userID and data in session
                    $this->create_login_session($response->records[0]->clientID,$firstName,$lastName);
                    $this->registration_info['status'] = 'created';
                }
            }
        }
        else{
            $this->registration_info['fields'] = array(
                'user_first_name' => $firstName,
                'user_last_name' => $lastName,
                'user_email' => $email,
                'user_username' => $username,
                'user_password' => $password,

            );
        }
    }

    public function user_login()
    {
        $ClientUsername = sanitize_text_field($_POST['ec_username']);
        $ClientPassword = sanitize_text_field($_POST['ec_password']);


        if(empty($ClientUsername) || empty($ClientPassword))
        {
            $this->session->setData('l_status', 'error');
        }
        else
        {
            // Send user login data to ERPLY
            $data = array(
                'username' => $ClientUsername,
                'password' => $ClientPassword
            );

            $response = $this->erply->sendRequest('verifyCustomerUser', $data);

            if($response->status->responseStatus == 'ok'){ // User logged in
                // Store userID and data in session
                #var_dump($response);

                $this->create_login_session($response->records[0]->clientID, $response->records[0]->clientFirstName, $response->records[0]->clientLastName,$response->records[0]->companyID,$response->records[0]->companyName);
            }
            else{ // Login failed
                $this->session->setData('l_status', 'error');
            }
        }
        $url = full_url();
        wp_redirect($url);
        exit;
    }

    public function user_logout()
    {
        $this->session->flush();
        $url = full_url();
        $url = preg_replace('~(\?|&)ec_action=[^&]*~','',$url);
        wp_redirect($url);
        exit;
    }

    private function create_login_session( $id, $fname, $lname, $companyID = 0, $companyName = '' )
    {
        $clientCheck = $this->session->getData('l_cd_clientID');

        if(!empty($clientCheck) && $clientCheck != $id)
            $this->session->flush();

        $this->session->setData('l_status', 'logged');
        $this->session->setData('l_cd_clientID', $id);
        $this->session->setData('l_cd_clientFirstName', $fname);
        $this->session->setData('l_cd_clientLastName', $lname);

        if (isset($companyID) && $companyID > 0) {
            $this->session->setData('l_cd_companyName', $companyName);
            $this->session->setData('l_cd_companyID', $companyID);
        }

    }

    public function get_user_orders()
    {
        if(ec_logged_in()){
            $companyID = $this->session->getData('l_cd_companyID');
            $data = array(
                'clientID' => !empty($companyID) ? $companyID : ec_logged_in(),
                'getRowsForAllInvoices' => 1
            );
            $response = $this->erply->sendRequest('getSalesDocuments', $data);
            if($response->status->errorCode === 0 ){ // User logged in
                if(!empty($response->records)){
                    $groups = array(
                        'ORDER' => array(
                            'name' => __('Orders',EC_TEXTDOMAIN),
                            'records' => array()
                            ),
                        'INVWAYBILL' => array(
                            'name' => __('Waybills',EC_TEXTDOMAIN),
                            'records' => array()
                        ),
                        'INVOICE' => array(
                            'name' => __('Invoices',EC_TEXTDOMAIN),
                            'records' => array()
                        )
                    );
                    foreach($response->records as $record){
                        if(array_key_exists($record->type,$groups))
                            array_push($groups[$record->type]['records'],$record);
                    }

                    // Remove empty groups
                    foreach($groups as $k => $v){
                        if(empty($v['records']))
                            unset($groups[$k]);
                    }

                    $return = array(
                        'status' => 'ok',
                        'groups' => $groups
                    );
                }
                else
                    $return = array(
                        'status' => 'empty',
                        'msg' => __('You don\'t have any orders yet.', EC_TEXTDOMAIN)
                    );
            }
            else{ // Login failed
                $return = array(
                    'status' => 'error',
                    'msg' => __('Query failed, please refresh the page!',EC_TEXTDOMAIN)
                );
                ECLog::erply('getSalesDocuments', $response->status->errorCode);
            }
        }
        else{
            $return = array(
                'status' => 'login',
                'msg' => __('Please login to see your orders!',EC_TEXTDOMAIN)
            );
        }
        return $return;
    }
}