<?php
class ERPLY
{

    public $clientCode = '';
    private $url = '';
    private $username = '';
    private $password = '';

    public function __construct($username, $password, $clientID)
    {
        $this->session = ECSessionHandler::get_instance();
        $this->username = $username;
        $this->password = $password;
        $this->clientCode = $clientID;
        $this->url        = 'https://' . $clientID . '.erply.com/api/';
    }

    // Sends POST request to API
    public function sendRequest($request, $parameters = array(), $json = false)
    {
        // Check if all nessecary parameters are set up
        if (!$this->url OR !$this->clientCode OR !$this->username OR !$this->password)
            return false;

        // Include clientcode and request name to POST parameters
        $parameters['request'] = $request;
        $parameters['version'] = "1.0";
        $parameters['clientCode'] = $this->clientCode;
        $parameters['responseType'] = 'JSON';

        // Get session KEY
        if ($request != "verifyUser") {
            $parameters['sessionKey'] = $this->getSessionKey($keyRequestResult);

            // Instead of a KEY we got an array which contains error code, let's return in
            if (!$parameters['sessionKey'])
                return $keyRequestResult;
        }

        // Prepare POST request
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->url);
        curl_setopt($ch, CURLOPT_HEADER, 1);
        curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_ANY);
        curl_setopt($ch, CURLOPT_SSLVERSION,3);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $parameters);
        // call POST request
        if (curl_exec($ch) === false)
            dd(curl_error($ch));
        // get response content
        $content = curl_multi_getcontent($ch);
        curl_close($ch);

        // remove headers
        list($header1, $header2, $body) = explode("\r\n\r\n", $content, 3);

        // return response body
        if($json)
            return $body;
        else
            return json_decode($body);
    }

    public function getSessionKey(&$result)
    {
        $sessionKey = $this->session->getData('EAPISessionKey_' . $this->username);
        $sessionKeyExpires = $this->session->getData('EAPISessionKeyExpires_' . $this->username);

        // Session KEY is active, return active KEY
        if ( !empty($sessionKey) && !empty($sessionKeyExpires) && $sessionKeyExpires > time() )
            return $sessionKey;
        else { // New session KEY must be obtained

            // Perform API request to get session KEY
            $response = $this->sendRequest("verifyUser",
                array("username" => $this->username, "password" => $this->password), true);

            // Json response into PHP array
            $response = json_decode($response, true);

            // Session KEY was successfully received
            if ($response['records'][0]['sessionKey']) {

                // Set session KEY in client session and set KEY expiration time
                $this->session->setData('EAPISessionKey_' . $this->username, $response['records'][0]['sessionKey']);
                $this->session->setData('EAPISessionKeyExpires_' . $this->username, time() + $response['records'][0]['sessionLength'] - 30);
                // Return obtained new session KEY
                return $response['records'][0]['sessionKey'];
            } // Session KEY was not received
            else {
                // Return API response which includes error code
                $this->session->unsetData('EAPISessionKey_' . $this->username);
                return false;
            }
        }
    }

}