<?php

class ECSync
{
    private static $instance = null;
    private $erply, $options, $lastSync, $depth;
    public $touched = array();
    public $inserted = array();
    public $forDelete = array();

    private function __construct()
    {
        $this->options = ECOptions::get_instance();
        $this->erply = new ERPLY($this->options->login['username'], $this->options->login['password'], $this->options->login['clientID']);
        $this->lastSync = !empty($this->options->main['lastSync']) ? $this->options->main['lastSync'] : null;
        $this->lastSync = !empty($this->options->main['region']) ? $this->options->main['lastSync'] : null;
        $this->lastSyncLog = !empty($this->options->main['lastSyncLog']) ? $this->options->main['lastSyncLog'] : null;
        $this->check_region();
    }

    public function check_region()
    {
        if (empty($this->options->main['region'])) {
            $response = $this->erply->sendRequest('getConfParameters', array());
            if ($response->status->errorCode == 0 && isset($response->records[0]->country)) {
                $mainOptions = (array)get_option('ec_main_settings');
                $mainOptions['region'] = $response->records[0]->country;
                update_option('ec_main_settings', $mainOptions);
            } else {
                ECLog::erply('getConfParameters', $response->status->errorCode);
            }
        }
    }

    private function walkCategories($args) {
        $record = $args['record'];
        $parentTerm = isset($args['parentTerm']) ? $args['parentTerm'] : null;
        $format = isset($args['format']) ? $args['format'] : 'html';

        $this->depth++;

        $key = array_search($record->id, $this->forDelete);
        if(!empty($key))
            unset($this->forDelete[$key]);

        $category = new ECSyncCategory();
        $response = $category->save_category($record, $parentTerm);

        $this->touched[] = $category->id;
        if (!empty($this->lastSync) && $response != 'updated')
            $this->inserted[] = $category->erplyID;

        $output = '';
        for ($i = 1; $i < $this->depth; $i++) {
            $output .= ' - ';
        }

        $action = $response == 'updated' ? __('Updated', EC_TEXTDOMAIN) : __('Inserted', EC_TEXTDOMAIN);
        $output .= $action . ' ' . $record->name;

        if($format == 'html')
            $output .= '</br>';
        else if($format == 'text')
            $output .= PHP_EOL;
        echo $output;

        if (!empty($record->subGroups))
            foreach ($record->subGroups as $subGroup) {
                $args = array(
                    'record' => $subGroup,
                    'parentTerm' => $category->id,
                    'format' => $format
                );
                $this->walkCategories($args);
            }

        $this->depth--;
    }

    public function test()
    {

        global $wpdb;
        $pref = $wpdb->prefix;
        $myrows = $wpdb->get_results(
            "select t.term_id, tm.meta_value as erply_id
            from {$pref}term_taxonomy tt
            join {$pref}terms t on t.term_id = tt.term_id
            join {$pref}ec_termmeta tm on t.term_id = tm.ec_term_id
            where taxonomy = 'product-category' and meta_key = 'erplyID';"
        );

        foreach($myrows as $row) {
            $this->forDelete[$row->term_id] = $row->erply_id;
        }

//        dd($this->forDelete);

        $groups = $this->erply->sendRequest('getProductGroups');

        if ($groups->status->errorCode === 0) {
            $records = $groups->records;
            if ($records) {
                foreach ($records as $record) {
                    $this->walkCategories(array('record' => $record));
                }
            }
        }
//        clean_term_cache($this->touched, 'product-category', true);
//        var_dump();

        foreach($this->forDelete as $term => $erplyID) {
            $result = ECSyncCategory::remove_category($term);
            if($result)
                echo 'Deleted category ' . $term .' (erplyID: ' . $erplyID . ')'.PHP_EOL;
        }

        dd($groups);



//        $catSync = new ECSyncCategory($this->lastSync);
//        $term = $catSync->check_if_category_exists(12, true);
//        dd($term);

//        $catSync->remove_category(11);


//        $output = array();
//        $i = 0;
//        foreach (new DirectoryIterator(EC_PATH . 'log') as $file) {
//            if ($file->isFile()) {
//                if($file->getExtension() === 'txt')
//                $output[$file->getCTime()] = $file->getFileName();
////                $output[] = file($file->getPathName());
//            }
//        }
//        krsort($output);
//        $first = key($output);
//        echo nl2br(file_get_contents(EC_PATH . 'log/' . $output[$first]));
//        dd($output);


//        $args = array(
//            'productID' => 94,
//            'getParameters' => 1,
//            'getRelatedProducts' => 1,
//            'getReplacementProducts' => 1,
//            'orderBy' => 'productID',
//            'orderByDir' => 'asc',
//            'getRelatedFiles' => 1
//        );
//        $res = $this->erply->sendRequest('getProducts', $args);
//
//        dd($res);
//
////        $product = new ECSyncProduct($res->records[0]);
////        $product->save();


//        global $wpdb;
//        $table_name = $wpdb->prefix . "ec_termmeta";
//        echo $table_name;
//        $res = $wpdb->get_results("SELECT meta_value FROM {$table_name} WHERE ec_term_id = 7 AND meta_key = 'images'");
//
////        $res = ec_term_meta(7, 'images');
//        dd($res);
    } // end test

    /**
     * @param $page
     * @param bool $json
     * @param array $argsERPLY
     * @return array|string
     */
    public function sync($page, $json = true, $argsERPLY = array())
    {
    	//unregister widgets
    	add_action('widgets_init', array($this, 'unregister_default_wp_widgets'), 1);
    	
        global $wpdb;
        $getNext = true;
        $timestamp = null;
        $postsPerPage = 15;

        // Retry option added
        $log = $this->lastSyncLog;

        if (isset($log) && empty($argsERPLY)) {
            // We have a broken import, asked for first page but different or same page in log.
            if (isset($log['page']) && $log['page'] > $page) {
                $page = $log['page'];
            }
        }

        if ($page != 1) {
            // Write the last page to log
            $mainOptions = (array) get_option('ec_main_settings');
            $mainOptions['lastSyncLog'] = array('page' => $page);
            update_option('ec_main_settings', $mainOptions);
        }

        $args = array(
            'recordsOnPage' => $postsPerPage,
            'pageNo' => $page,
            'getParameters' => 1,
            'getRelatedProducts' => 1,
            'getReplacementProducts' => 1,
            'orderBy' => 'productID',
            'orderByDir' => 'asc',
            'getRelatedFiles' => 1
        );


        if (!empty($argsERPLY)) {
            foreach ($argsERPLY as $argKey => $arg) {
                $args[$argKey] = $arg;
            }
        }

        if ($this->lastSync && empty($argsERPLY))
            $args['changedSince'] = $this->lastSync;

        $response = $this->erply->sendRequest('getProducts', $args);
        $result = array();

        if ($response->status->errorCode === 0) {
            if ($timestamp == null)
                $timestamp = $response->status->requestUnixTime;
            if (count($response->records) > 0) {
                $counter = 1 + (($page - 1) * $postsPerPage);
                foreach ($response->records as $product) {
                    $product = new ECSyncProduct($product);
                    $result[] = '#' . $counter++ . ': ' . $product->save();
                }
            } else {
                if ($this->lastSync){
                    $mainOptions = (array)get_option('ec_main_settings');
                    $mainOptions['lastSync'] = $timestamp;
                    unset($mainOptions['lastSyncLog']);
                    update_option('ec_main_settings', $mainOptions);
                    die(json_encode(array('status' => 'error', 'msg' => 'No product updates since ' . date('Y-m-d H:i:s', $this->lastSync) . PHP_EOL)));
                }

                else if ($page == 1)
                    die(json_encode(array('status' => 'error', 'msg' => 'No eligible products found.' . PHP_EOL, 'data' => $response)));
            }
        } else {
            ECLog::erply('getProducts', $response->status->errorCode);
            if ($json)
                die(json_encode(array('status' => 'error', 'msg' => 'ERPLY error: ' . $response->status->errorCode . PHP_EOL)));
            else
                return 'ERPLY error: ' . $response->status->errorCode;
        }


        if (count($response->records) < $postsPerPage) {
            $getNext = false;
            if (function_exists('w3tc_dbcache_flush')) { w3tc_dbcache_flush(); }
            if (function_exists('w3tc_pgcache_flush')) { w3tc_pgcache_flush(); }
            if (empty($argsERPLY)) {
                $mainOptions = (array) get_option('ec_main_settings');
                $mainOptions['lastSync'] = $timestamp;
                unset($mainOptions['lastSyncLog']);
                update_option('ec_main_settings', $mainOptions);
            }
        }


        $result = array(
            'askNext' => $getNext,
            'nextPage' => ++$page,
            'status' => 'ok',
            'result' => $result,
            'queries' => $wpdb->queries,
            'totalRecords' => $response->status->recordsTotal,
            'recordsInResponse' => $response->status->recordsInResponse,
            'recordsPerPage' => $postsPerPage,
            'unixStamp' => $timestamp,
            'msg' => $getNext ? '' : __('Product sync finished', EC_TEXTDOMAIN) . PHP_EOL
        );

        if ($json)
            die(json_encode($result));
        else
            return $result;
    } // end sync()


    public function unregister_default_wp_widgets() {
    	unregister_widget('WP_Widget_Pages');
    	unregister_widget('WP_Widget_Calendar');
    	unregister_widget('WP_Widget_Archives');
    	unregister_widget('WP_Widget_Links');
    	unregister_widget('WP_Widget_Meta');
    	unregister_widget('WP_Widget_Search');
    	unregister_widget('WP_Widget_Text');
    	unregister_widget('WP_Widget_Categories');
    	unregister_widget('WP_Widget_Recent_Posts');
    	unregister_widget('WP_Widget_Recent_Comments');
    	unregister_widget('WP_Widget_RSS');
    	unregister_widget('WP_Widget_Tag_Cloud');
    	unregister_widget('WP_Nav_Menu_Widget');
    }
    
    public function check_updates()
    {
        var_dump($this->lastSync);
        $changes = $this->erply->sendRequest('getChangedDataSince', array('changedSince' => $this->lastSync));

        if ($changes->status->errorCode === 0) {
            foreach ($changes->records as $table) {
                switch ($table->tableName) {
                    case 'products':
                        if ($table->updated != 0)
                            $this->sync_products();
                        if ($table->deleted != 0)
                            $this->sync_deleted_products();
                        break;
                }
            }
            var_dump($changes);
        } else {
            ECLog::erply('getChangedDataSince', $changes->status->errorCode);
        }
    } // end check_updates()

    public function sync_categories()
    {

        global $wpdb;
        $pref = $wpdb->prefix;
        $myrows = $wpdb->get_results(
            "select t.term_id, tm.meta_value as erply_id
            from {$pref}term_taxonomy tt
            join {$pref}terms t on t.term_id = tt.term_id
            join {$pref}ec_termmeta tm on t.term_id = tm.ec_term_id
            where taxonomy = 'product-category' and meta_key = 'erplyID';"
        );

        foreach($myrows as $row) {
            $this->forDelete[$row->term_id] = $row->erply_id;
        }

        $groups = $this->erply->sendRequest('getProductGroups');

        if ($groups->status->errorCode === 0) {
            $records = $groups->records;
            if ($records) {
                foreach ($records as $record) {
                    $this->walkCategories(array('record' => $record, 'format' => 'text'));
                }

                // Delete categories
                foreach($this->forDelete as $term => $erplyID) {
                    $result = ECSyncCategory::remove_category($term);
                    if($result)
                        echo 'Deleted category ' . $term .' (erplyID: ' . $erplyID . ')'.PHP_EOL;
                }

                //Clear term cache
                if (!empty($this->touched))
                    clean_term_cache($this->touched);

                if (!empty($this->inserted)) {
                    foreach ($this->inserted as $groupID) {
                        $getNext = true;
                        $page = 1;
                        set_time_limit(1000);
                        while ($getNext) {
                            $res = $this->sync($page, false, array('groupID' => $groupID));

                            echo 'Page: #' . $page . ' (queries: ' . count($res['queries']) . ')</br>';

                            $getNext = isset($res['askNext']) ? $res['askNext'] : false;
                            $page = isset($res['nextPage']) ? $res['nextPage'] : null;

                            foreach ($res['result'] as $productRes) {
                                echo $productRes;
                            }
                        }
                    }

                }
            } else
                echo __('No records found.', EC_TEXTDOMAIN);
        } else {
            ECLog::erply('getProductGroups', $groups->status->errorCode);
            echo 'ERPLY error: ' . $groups->status->errorCode . PHP_EOL;
        }
    } // end sync_categories()

    public static function get_instance()
    {
        if (self::$instance == null)
            self::$instance = new self;
        return self::$instance;
    } // end get_instance
}