<?php
class ECSyncCategory
{
    public $id, $erplyID, $name;

    public function save_category($record, $parentTerm = null, $fixName = false)
    {
        $termArgs = array();
        $this->erplyID = $record->id;
        $this->name = $record->name;
        $termObj = $this->check_if_category_exists($record->id, true);
        $termID = $termObj->term_id;
        if (empty($record->name)) return array('status' => 'error', 'msg' => __('Category name can not be empty.', EC_TEXTDOMAIN) . PHP_EOL);

        if (!empty($termID)) {
            if ($record->showInWebshop != 1) {
                if (self::remove_category($termID)) {
                    echo __('Deleted category ', EC_TEXTDOMAIN) . $record->name . PHP_EOL;
                    return array('status' => 'error', 'msg' => __('Deleted category ', EC_TEXTDOMAIN) . $record->name);
                }
            }
        } else if ($record->showInWebshop != 1) return array('status' => 'error', 'msg' => __('Disabled for webshop.', EC_TEXTDOMAIN) . PHP_EOL);

        $name = $fixName ? $record->name . ' (' . $record->id . ')' : $record->name;

        if (!empty($parentTerm))
            $termArgs['parent'] = $parentTerm;
        else
            if (isset($termObj->parent))
                $termArgs['parent'] = 0;


        if (!empty($termID)) {
            $termArgs['name'] = $name;
            $term = wp_update_term($termID, 'product-category', $termArgs);
        } else {
            $term = wp_insert_term(
                $name,
                'product-category',
                $termArgs
            );
        }

        if ($term instanceof WP_Error) {
            if (!$fixName)
                $this->save_category($record, $parentTerm, true);
        } else {
            $update = !empty($termID) ? true : false;
            $this->set_category_attributes($term['term_id'], $record, $update);
            $this->set_category_images($term['term_id'], $record, $update);
            $this->id = $term['term_id'];
            return $update ? 'updated' : 'inserted';
        }
        return null;
    } // end save_category()

    public static function remove_category($termID)
    {
        $products = get_objects_in_term($termID, 'product-category');
        foreach ($products as $postID) {
            wp_delete_post($postID, true);
        }
        remove_ec_term_meta($termID);
        $result = wp_delete_term($termID, 'product-category');
//        dd($products);

        return !empty($result);
    } // end remove_category()

    public function check_if_category_exists($erplyGroupID, $returnObj = false)
    {
        global $wpdb;
        $tableTerms = $wpdb->prefix . 'terms';
        $tableTermMeta = $wpdb->prefix . 'ec_termmeta';
        $tableTermTaxonomy = $wpdb->prefix . 'term_taxonomy';

        $sql = "SELECT * FROM {$tableTerms} t
            JOIN {$tableTermMeta} tm ON tm.ec_term_id = t.term_id
            JOIN {$tableTermTaxonomy} tt ON tt.term_id = t.term_id
            WHERE tm.meta_key = 'erplyID' AND tm.meta_value = {$erplyGroupID}";
        $terms = $wpdb->get_results($sql);
        if (is_array($terms) && isset($terms[0]) && isset($terms[0]->term_id) && is_numeric($terms[0]->term_id))
            if ($returnObj)
                return $terms[0];
            else
                return $terms[0]->term_id;
        return null;
    } // end check_if_category_exists()

    private function set_category_images($termID, $record, $update = false)
    {
        $exists = exists_ec_term_meta($termID, 'images');
        if (!isset($record->images)) {
            if ($exists)
                remove_ec_term_meta($termID, 'images');
            return;
        } else $images = $record->images;

        foreach ($images as $key => $img) {
            if (!check_image($img->thumbURL))
                unset($images[$key]);
        }

        if (!empty($images)) {
            if ($exists) update_ec_term_meta($termID, 'images', serialize($images));
            else add_ec_term_meta($termID, 'images', serialize($images));
        } else {
            if ($update && $exists)
                remove_ec_term_meta($termID, 'images');
            return;
        }
    } // end set_category_images

    public function set_category_attributes($termID, $erplyGroup, $update = false)
    {
        $metaFields = array(
            'erplyID' => $erplyGroup->id,
        );

        if ($update) {
            foreach ($metaFields as $key => $value) {
                update_ec_term_meta($termID, $key, $value);
            }
        } else {
            foreach ($metaFields as $key => $value) {
                add_ec_term_meta($termID, $key, $value);
            }
        }
    } // end set_category_attributes()
}