<?php 

/**
 * Product sync class
 */

class ECSyncProduct 
{
	private $record;
	static $categoryCache = array();

	public function __construct($record)
	{
		$this->record = $record;
		$this->options = ECOptions::get_instance();
	} // end __construct()

	public function save()
	{	
        //get post and category id
		$postID = $this->get_matching_post_id($this->record->productID);
		$categoryID = $this->get_matching_category_id($this->record->groupID);
		$parentPostNeeded = (isset($this->record->parentProductID) && $this->record->parentProductID) ? true : false;
		$parentPostId = $parentPostNeeded ? $this->get_matching_post_id($this->record->parentProductID) : false;
		$update = ($postID) ? true : false;

		//check if eligible for webshop
		$eligible = true;
        if (
        	$this->record->displayedInWebshop != 1 || 									/*not displayed in web shop*/
        	empty($categoryID) ||  														/*category not found*/
        	!in_array($this->record->status, array('ACTIVE', 'NO_LONGER_ORDERED')) || 	/*bad status*/
        	($parentPostNeeded && !$parentPostId) 										/*missing parent post for matrix variation*/
        ) $eligible = false;

        //handle non-eligible product
       	if($update && !$eligible) {
        	if($this->remove($postID)) return __('Deleted product (erplyID:',EC_TEXTDOMAIN). ' ' . $this->record->productID .'): ' . $this->record->name;
        	else return __('Couldn\'t remove product: ', EC_TEXTDOMAIN) . $this->record->productID;
        } elseif(!$update && !$eligible) return __('Product not eligible: ', EC_TEXTDOMAIN) . $this->record->productID;
        
        //set post params
        if (!empty($postID)) $postData['ID'] = $postID;
        if($parentPostId) $postData['post_parent'] = $parentPostId;
        $postData['comment_status']  = !empty($this->options->main['productReviews']) ? 'open' : 'closed';
        $postData['post_author']  = 1;
        $postData['post_content']  = $this->record->longdesc;
        $postData['post_excerpt']  = $this->record->description;
        $postData['post_status']  = 'publish';
        $postData['post_title']  = $this->record->name;
        $postData['post_type']  = 'product';
        $postData['post_name'] = $this->slug_check($this->record->name, $this->record->productID);

        //insert
        $post = wp_insert_post($postData, true);

        //fail on error
        if ($post instanceof WP_Error) {
            $errorMessages = $post->get_error_messages();
            return $errorMessages;
        }

        //  Set Product Attributes
      	$this->set_product_attributes($post, $this->record, $update);
			
        // Set product category
		$this->set_product_category($post, $categoryID);

       	// Set product images
        if(isset($this->record->images)) $this->set_product_images($post, $this->record->images, $update);
            
        // Set product files
		if(isset($this->record->relatedFiles)) $this->set_product_files($post, $this->record->relatedFiles, $update);
        
		//return result as text
        $action = ($postID) ? __('Updated', EC_TEXTDOMAIN) : __('Inserted', EC_TEXTDOMAIN);
        return $action . ' product (erplyID: '. $this->record->productID .'): ' . $this->record->name;

	} // end save()


	private function get_matching_post_id($erplyID)
	{
		global $wpdb;
        
        //make the query
        $erplyID = intval($erplyID);
        $q = "SELECT post_id FROM {$wpdb->postmeta} WHERE meta_key = 'erplyID' AND meta_value = '$erplyID'"; /*This does not include post type so possibly error prone*/
        $results = $wpdb->get_results($q, ARRAY_A ); 
        
        //return id
        return (is_array($results) && isset($results[0])) ? $results[0]['post_id'] : null;
	} // end get_matching_post_id()

	public function slug_check($productName, $productID)
	{
		global $wpdb;
        
		//make a slug
		$slug = preg_replace('/[^A-Za-z0-9-]+/', '-', $productName);
		
		//check if slug exists
		$tambleName = $wpdb->prefix . 'posts';
		$sql = "SELECT post_name FROM {$tambleName} WHERE post_name = '{$slug}' AND post_type IN ( 'page', 'product' ) AND ID != 0 AND post_parent = 0 LIMIT 1";
		$existingSlug = $wpdb->get_var( $sql );
		
		//use product id also if slug already exists
		if($existingSlug) $slug .=  '-'.$productID;
		
		//return
		return $slug;
	} // end slug_check()

	private function get_matching_category_id($erplyGroupID)
	{
		global $wpdb;
		
		//fetch from db if not in cache
		if(!isset(ECSyncProduct::$categoryCache[$erplyGroupID])) {
			$table_name = $wpdb->prefix . "ec_termmeta";
			$results = $wpdb->get_results("SELECT ec_term_id FROM {$table_name} WHERE meta_key = 'erplyID' AND meta_value = {$erplyGroupID}");
			$categoryId = (isset($results) && isset($results[0])) ? $results[0]->ec_term_id : null;
			ECSyncProduct::$categoryCache[$erplyGroupID] = $categoryId;
		}
		
		return ECSyncProduct::$categoryCache[$erplyGroupID];
	} // end get_matching_category_id()

    private function set_product_images($productID, $images, $update = false)
    {
        
        foreach ($images as $key => $img) {
            if(!check_image($img->thumbURL))
                unset($images[$key]);
        }

        if(!empty($images)) {
            if($update) 
                update_post_meta( $productID, 'images', serialize($images) );
            else 
                add_post_meta( $productID, 'images', serialize($images) );
        } else {
            if ($update)
                delete_post_meta($productID, 'images');
        }
    } // end set_product_images
	
    private function set_product_files($productID, $files, $update = false)
    {
        if(!empty($files)) {
            if($update) 
                update_post_meta( $productID, 'files', serialize($files) );
            else 
                add_post_meta( $productID, 'files', serialize($files) );
        } else {
            if ($update)
                delete_post_meta($productID, 'files');
        }
    } // end set_product_images

    private function set_product_category($productID, $categoryID)
    {
        wp_set_post_terms($productID, $categoryID, 'product-category');
    } // end set_product_category

    private function set_product_attributes($productID, $erplyRow, $update = false)
    {
        $metaFields = array(
            'erplyID' => $erplyRow->productID,
            'price' => $erplyRow->price,
            'priceWithVat' => $erplyRow->priceWithVat,
            'code' => $erplyRow->code,
            'code2' => $erplyRow->code2,
            'code3' => $erplyRow->code3,
            'length' => $erplyRow->length,
            'width' => $erplyRow->width,
            'height' => $erplyRow->height,
            'series' => $erplyRow->categoryName,
            'unit_name' => $erplyRow->unitName,
            'brand_name' => $erplyRow->brandName,
            'country_of_origin_id' => $erplyRow->countryOfOriginID,
            'type' => $erplyRow->type
        );
        
        if(isset($erplyRow->replacementProducts)) {
            if(!$update)
                add_post_meta( $productID, 'replacementProducts', serialize($erplyRow->replacementProducts));
            else 
                $metaFields['replacementProducts'] = serialize($erplyRow->replacementProducts);                

        }
        if(isset($erplyRow->relatedProducts)) {
            if(!$update)
                add_post_meta( $productID, 'relatedProducts', serialize($erplyRow->relatedProducts));
            else 
                $metaFields['relatedProducts'] = serialize($erplyRow->relatedProducts);
        }     

        if(isset($erplyRow->parameters)){
            $parameters = array();
            foreach ($erplyRow->parameters as $param) {
                $parameters[] = array(
                    'name' => $param->parameterName,
                    'value' => $param->parameterValue,
                    'type' => $param->parameterType
                );
            }
            if(!$update)
                add_post_meta( $productID, 'parameters', serialize($parameters));
            else 
                $metaFields['parameters'] = serialize($parameters);
        } 

        if($update) {
            foreach ($metaFields as $key => $value) {
                update_post_meta( $productID, $key, $value );
            }
        } else 
            $this->insert_post_meta($metaFields, $productID);
       
    } // end set_product_attributes

    private function remove($ID)
    {
        $res = wp_delete_post( $ID );
        
        return !$res ? false : true;
    } // end remove_post     

    private function insert_post_meta($fields, $postID)
    {
        global $wpdb;
        foreach ($fields as $key => $value) {
            $meta_values[] = $wpdb->prepare('(NULL, %d, %s, %s)', $postID, $key, $value);
        }
        $values = implode(', ', $meta_values);
        $res = $wpdb->query("INSERT INTO $wpdb->postmeta (meta_id, post_id, meta_key, meta_value) VALUES $values");
    } // end insert_post_meta()
}