<?php

class ECFrontProducts_Widget extends WP_Widget {

	/**
	 * Register Widget
	 */
	public function __construct() {

		parent::__construct(
	 		'ec_front_products_widget', // Base ID
            __('eCommerce - Front Categories', EC_TEXTDOMAIN ), // Name
			array( 'description' => __( 'Lists top level categories for frontpage', EC_TEXTDOMAIN ) ) // Args
		);
	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {

		extract( $args );

        $title           = apply_filters( 'widget_title', $instance['title'] );
        $num_posts       = apply_filters( 'widget_number_of_posts', $instance['num_posts'] );

        echo $before_widget;

        // Print out the main categories
        $args = array(
            'parent' => 0,
            'number' => $num_posts,
            'hide_empty' => 0
        );
        $posts = get_terms('product-category',$args);

        echo '<div class="ec_front_cat_container">';
        if ( ! empty( $title ) )
            echo $before_title . $title . $after_title;

        echo '<ul class="ec_front_categories">';

        foreach($posts as $post):
            $img = ec_term_meta($post->term_id,'images');
        	$img = ($img) ? get_image($img,'small') : get_placeholder_image('small');

            $loop_html = '<li><div class="ec_category_content">
                        <a class="ec_front_thumb" href="'. get_term_link( $post->slug, 'product-category' ) .'">'. $img .'</a>
                        <p><a href="'. get_term_link( $post->slug, 'product-category' ) .'">'. $post->name .'</a></p>
                        </div></li>';
            echo apply_filters( 'ec_front_categories_loop', $loop_html, $post );
        endforeach;
        echo '</ul></div>';

		echo $after_widget;
	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {
		if ( isset( $instance[ 'title' ] ) ) 
			$title = $instance[ 'title' ];
		else 
			$title = __( 'Catalog', EC_TEXTDOMAIN );

		if ( isset( $instance[ 'num_posts' ] ) ) 
			$num_posts = $instance[ 'num_posts' ];
		else 
			$num_posts = 5;
		
		?>
		<p>
		<label for="<?php echo $this->get_field_name( 'title' ); ?>"><?php _e( 'Title:', EC_TEXTDOMAIN ); ?></label> 
		<input 
			class="widefat" 
			id="<?php echo $this->get_field_id( 'title' ); ?>" 
			name="<?php echo $this->get_field_name( 'title' ); ?>" 
			type="text" 
			value="<?php echo esc_attr( $title ); ?>" />
		</p>
		<label for="<?php echo $this->get_field_name( 'num_posts' ); ?>"><?php _e( 'Number of categories to show:', EC_TEXTDOMAIN ); ?></label> 
		<input 
			class="widefat" 
			id="<?php echo $this->get_field_id( 'num_posts' ); ?>" 
			name="<?php echo $this->get_field_name( 'num_posts' ); ?>" 
			type="text" 
			value="<?php echo esc_attr( $num_posts ); ?>" />
		</p>
		<?php 
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		// $defaults = array( 'title' => __('Example', 'example'), 'name' => __('John Doe', 'example'), 'sex' => 'male', 'show_sex' => true );
		// $instance = wp_parse_args( (array) $instance, $defaults );

		$instance['title'] = ( !empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
        $instance['num_posts'] = ( !empty( $new_instance['num_posts'] ) ) ? strip_tags( $new_instance['num_posts'] ) : 5;

		return $instance;
	}
    
}