<?php
class ECLogin_Widget extends WP_Widget
{

	/**
	 * Register Widget
	 */
	public function __construct()
    {
        $this->options = ECOptions::get_instance();
        $this->session = ECSessionHandler::get_instance();

        $loginStatus = $this->session->getData('l_status');

        if(!empty($loginStatus) && $loginStatus == 'error') {
            $this->session->unsetData('l_status');
//            $this->session->unsetData('l_seen');
            $this->session->unsetData('l_cd_clientID');
            $this->session->unsetData('l_cd_clientFirstName');
            $this->session->unsetData('l_cd_clientLastName');
            $this->session->unsetData('l_cd_companyName');
            $this->session->unsetData('l_cd_companyID');
            $this->session->unsetData('l_cd_sa_addressID');
            $this->session->unsetData('l_cd_sa_street');
        }

        register_nav_menu( 'ec_login_widget', __( 'Login Widget Menu', EC_TEXTDOMAIN ) );

        parent::__construct(
	 		'ec_login', // Base ID
			__('eCommerce - User Login',EC_TEXTDOMAIN), // Name
			array( 'description' => __( 'User login form', EC_TEXTDOMAIN ) ) // Args
		);
	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {
		extract( $args );
		$option = get_option( 'ec_main_settings' );
		$allowed = isset($option['allowLogin']);
		$title = apply_filters( 'widget_title', $instance['title'] );
		if(!$allowed) return;
		echo $before_widget;
		if ( ! empty( $title ) )
			echo $before_title . $title . $after_title;
		$this->showForm($instance);
		echo $after_widget;
	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {
		if ( isset( $instance[ 'title' ] ) ) {
			$title = $instance[ 'title' ];
		}
		else {
			$title = __( 'User Login', EC_TEXTDOMAIN );
		}
		?>
		<p>
		<label for="<?php echo $this->get_field_name( 'title' ); ?>"><?php _e( 'Title:',EC_TEXTDOMAIN ); ?></label>
		<input
			class="widefat"
			id="<?php echo $this->get_field_id( 'title' ); ?>"
			name="<?php echo $this->get_field_name( 'title' ); ?>"
			type="text"
			value="<?php echo esc_attr( $title ); ?>" />
		</p>
		<?php
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = array();
		$instance['title'] = ( !empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';

		return $instance;
	}

	public function showForm($instance)
	{
        $currentURL = full_url();

        $mainOptions = get_option('ec_main_settings');
        $pageOptions = get_option('ec_pages_settings');
        $registerPageID = $pageOptions['registerPage'];

		if(ec_logged_in()) : // User is logged in display logout button
		    $separator = ( empty( $_GET ) ) ? '?' : '&';
            $logOutURL = $currentURL. $separator.'ec_action=ec_logout';
		?>
		    <p class="ec_login_welcome"><?php  $this->loggedMessage(); ?></p>
		    <?php do_action( 'ec_login_after_message', $this->session->getAllData() ); ?>
		    <?php if( $this->session->getData('l_cd_companyID') ): ?>
		    	<p class="ec_login_company_name"><?php echo $this->session->getData('l_cd_companyName'); ?></p>
			<?php endif; ?>
		    
	    	<?php has_nav_menu('ec_login_widget') ? wp_nav_menu( array( 'theme_location' => 'ec_login_widget' ) ) : ''; ?>
            <a href="<?php echo $logOutURL; ?>"><?php _e('Log out',EC_TEXTDOMAIN) ?></a>
            
		<?php
		else:
		    // Display login form
		    if(isset($this->loginData['status']) && $this->loginData['status'] == 'error')
		        echo '<p class="ec_error">'.__('Login failed! Check your username and password',EC_TEXTDOMAIN).'</p>';

		?>
		<form action="<?php echo $currentURL ?>" method="post">
			<div class="ec_user_login">
                <div class="row">
                    <label for="ec_username"><?php _e('Username', EC_TEXTDOMAIN); ?></label>
                    <input type="text" name="ec_username" id="ec_username">
                </div>
                <div class="row">
                    <label for="ec_password"><?php _e('Password', EC_TEXTDOMAIN); ?></label>
                    <input type="password" name="ec_password" id="ec_password">
                </div>
                <div class="row">
                    <input type="hidden" name="ec_action" value="ec_login">
                    <input type="submit" value="<?php _e('Login', EC_TEXTDOMAIN); ?>">
                    <?php if(isset($mainOptions['allowRegistration']) && !isset($instance['hideRegistration'])): ?>
			            <p class="ec_login_register"><a href="<?php echo get_permalink($registerPageID); ?>"><?php _e('Create an account', EC_TEXTDOMAIN) ?></a></p>
			        <?php endif; ?>
                </div>
			</div>
		</form>
        
		<?php
		endif;
	}

    public function loggedMessage(){
        $find = array('/%firstname%/i', '/%lastname%/i');
        $replace = array($this->session->getData('l_cd_clientFirstName'), $this->session->getData('l_cd_clientLastName'));

        $welcome = $this->options->main['welcomeMessage'];

        $welcome = preg_replace($find, $replace, $welcome);
        $welcome = preg_replace("/%.*%/", "", $welcome);
        echo $welcome;
    }

}