<?php
/*
Plugin Name: eCommerce Shopping Cart by Inventory.com
Plugin URI: http://www.inventory.com/ecommerce-shopping-cart/
Description: eCommerce Shopping Cart plugin backed by a powerful order and inventory management service.
Version: 1.0.2
Author: Inventory.com
Author URI: http://www.inventory.com
License: GPL3
*/

require_once(plugin_dir_path(__FILE__) . '/config.php');


class ERPLYCart extends ECBase
{
    function __construct()
    {
        parent::__construct();
        // Setup deactivation for cleaning up after plugin
        register_deactivation_hook(__FILE__, array($this, 'deactivate'));
        register_activation_hook(__FILE__, array($this, 'activate'));

        add_filter('template_include', array($this, 'ec_template_include'), 1, 1);
    }

    /*
    ======================================================================================
        Activate & Deactivate
    ======================================================================================
    */

    /**
     * Fired when plugin is activated
     */
    public function activate()
    {
    	//check for user rights
        if (!current_user_can('activate_plugins')) return;
   
        //security check
        $plugin = isset($_REQUEST['plugin']) ? $_REQUEST['plugin'] : '';
        check_admin_referer("activate-plugin_{$plugin}");

        //create custom tables
        $this->create_termmeta_table();
        $this->create_sessiondata_table();
        
        //add pages
        $this->populate_pages();
		
		// Display activation notice and congiguration button
		$notices= get_option('deferred_admin_notices', array());
		$notices[]= array('Inventory.com plugin is activated. Some new pages were created for cart, checkout, terms of service etc. Also some new widgets were added for customer login, cart, product categories etc. You can set these up later. <strong>Right now the next step is to <a href="admin.php?page=erply-cart-settings"> configure your Inventory.com account here</a></strong>.','notice');
		update_option('deferred_admin_notices', $notices);
		
    } // activate()

    /**
     * Fired when plugin is deactivated
     */
    public function deactivate()
    {
    	//check for user rights
        if (!current_user_can('activate_plugins')) return;
        
        //security check
        $plugin = isset($_REQUEST['plugin']) ? $_REQUEST['plugin'] : '';
        check_admin_referer("deactivate-plugin_{$plugin}");

    } // end deactivate()


    public function route()
    {
        if (!empty($_REQUEST['action'])) {
            switch ($_REQUEST['action']) {
                case 'sync':
                    $this->synchronize();
                    break;
                case 'flush':
                    $this->flush();
                    break;
                case 'test':
                    $this->test();
            }
        } else $this->view_index();
    }

    public function view_index()
    {
        $view = new View('main.tpl.php');
        $view->log = isset($this->options->main['lastSyncLog']) ? $this->options->main['lastSyncLog'] : '';
        $view->publish();
    }

    public function synchronize()
    {
        if (empty($this->options->login['username']) || empty($this->options->login['password']) || empty($this->options->login['clientID'])) {
            echo __('Inventory.com account information not set. Go to eCommerce -> Settings and follow the instructions there.', EC_TEXTDOMAIN);
        } else {
            $sync = ECSync::get_instance();
            $sync->cat_sync();
        }
    }

    public function test()
    {
        $sync = ECSync::get_instance();
        $sync->test();
    } // end test()

    function ec_template_include($template)
    {
        global $wp_query;
        //dd($wp_query->post);
        if (isset($wp_query->post->post_parent) && 0 != $wp_query->post->post_parent) {
            wp_redirect(get_permalink($wp_query->post->post_parent) . '#variation-' . $wp_query->post->ID);
            exit;
        }

        if ( $wp_query->query_vars['post_type'] === 'product' && '' === ( $template = locate_template( array('single-product.php') ) ) ) {
            $template = EC_PATH . '/templates/single-product.php';
        }

        if( is_tax( 'product-category' ) && '' === ( $template = locate_template( array('taxonomy-product-category.php') ) ) )
            $template = EC_PATH . '/templates/taxonomy-product-category.php';
        return $template;
    }

    function flush()
    {
        $query = new WP_Query(array(
            'post_type' => array('product'),
            'posts_per_page' => -1
        ));

        if ($query->have_posts()) {
            echo '<ul>';
            while ($query->have_posts()) {
                $query->the_post();
                $name = get_the_title();
//                $deleted = true;
                $deleted = wp_delete_post(get_the_ID(), 'category');
                echo "<li>";
                echo $name;
                if ($deleted)
                    echo ' (deleted)';
                else
                    echo ' (error)';
                echo "</li>";
            }
            echo '</ul>';
        }

        wp_reset_postdata();


        $terms = get_terms('product-category', 'hide_empty=0');
        $count = count($terms);
        echo '<h2>Categories ( ' . $count . ' )</h2>';
        if ($count > 0) {
            echo "<ul>";
            foreach ($terms as $term) {
                echo "<li>";
                echo $term->name;
//                $deleted = true;
                $deleted = wp_delete_term($term->term_id, 'product-category');
                if ($deleted){
                    echo ' (deleted)';
                    remove_ec_term_meta($term->term_id);
                } else
                    echo ' (error)';
                echo "</li>";
            }
            echo "</ul>";
        }

        $mainOptions = (array)get_option('ec_main_settings');
        unset($mainOptions['lastSync']);
        unset($mainOptions['lastSyncLog']);
        update_option('ec_main_settings', $mainOptions);
    }
}

$ERPLYCart = new ERPLYCart();


